from enum import Enum


class DeploymentStatus(Enum):
    UPGRADE_FAILED = - 10
    UNKNOWN = 10
    MAINTENANCE_MODE_ENABLED = 20
    UPGRADE_STARTED = 30
    SYSTEM_BACKUP_DATA_READY = 40
    SYSTEM_SETTINGS_BACKUP_READY = 50
    SYSTEM_BACKUP_READY = 60
    MIGRATION_STARTED = 70
    MIGRATION_FINISHED = 80
    UPGRADE_FINISHED = 90
    UPGRADE_FAILED_RECOVERED = 95
    STARTED = 100
    STOPPED = 110

    def is_in_progress(self):
        return DeploymentStatus.MAINTENANCE_MODE_ENABLED.value <= self.value < DeploymentStatus.UPGRADE_FINISHED.value


class UpgradeContext:

    def __init__(self, run_id: str, license_token: str, current_version: str, target_version: str, extra_vars: dict,
                 deployment_client, dry_run: bool):
        assert run_id is not None, "run_id is None"

        self._stage = DeploymentStatus.UNKNOWN
        self._run_id = run_id
        self._license_token = license_token
        self._deployment_api = deployment_client
        self._current_version = current_version
        self._target_version = target_version
        self._extra_vars = (extra_vars or dict()).copy()
        self.dry_run = dry_run

    def set_status(self, stage_name: DeploymentStatus):
        if not self.dry_run:
            self._deployment_api.set_deployment_status(self, status=stage_name)

    def update(self, stage_name: DeploymentStatus, new_version):
        if not self.dry_run:
            self._deployment_api.set_deployment_status(self, status=stage_name, current_version=new_version)

    def status(self):
        resp = self._deployment_api.get_deployment(self._license_token)
        status_ = resp['status']
        return DeploymentStatus[status_.upper()]

    def run_id(self):
        return self._run_id

    def extra_vars(self):
        return self._extra_vars.copy()

    def docker_password(self):
        resp = self._deployment_api.get_deployment(self._license_token)
        if resp:
            return resp['dockerRegistryPassword']

    def azure_storage_key(self):
        resp = self._deployment_api.get_deployment(self._license_token)
        if resp:
            return resp['azureRegistryPassword']

    def current_version(self):
        return self._current_version

    def target_version(self):
        return self._target_version

    def license_token(self):
        return self._license_token
