import json
import dateutil.parser
import requests

import socket

from bottle import Bottle
from bottle import HTTPResponse
from bottle import request, response
from compose.project import NoSuchService
from docker import errors
import conduit_config
from requests import exceptions

from typing import List

from elasticsearch import Elasticsearch, ElasticsearchException, ConnectionTimeout, ConnectionError, NotFoundError, TransportError
from common import UpgradeContext, DeploymentStatus
from backup_service import BackupService, RepositoryVerificationError, RepositoryConcurrencyError
from conduit_cluster_manager import ConduitClusterManager
from conduit_disk_monitoring import VMInfo, DiskNotFoundException, DigitalInfoUnitOfMeasure
from infrastructure_api import DeploymentApiClient
from log_config import logger
from service_managment import DockerComposeServiceManager, ServiceId

from upgrade_service import UpgradeService, UpgradeError, ConcurrentUpgradeAttempt
from certificate_manager import CertificateInstallationError
app = Bottle()

server_ip = conduit_config.get("service.server.ip")
server_port = int(conduit_config.get("service.server.port"))

deployment_api_url = conduit_config.get("system.deployment_api.url")
deploy_service_api = DeploymentApiClient(deployment_api_uri=deployment_api_url,
                                         auth_token=conduit_config.get_deployment_token())

es_client_ = Elasticsearch(hosts=conduit_config.get_elasticsearch_host(), timeout=60)

_service_management = DockerComposeServiceManager(base_dir=conduit_config.get_docker_compose_dir(),
                                                  filenames=conduit_config.get_docker_compose_files())
_backup_service = BackupService(es_client_, _service_management)
_conduit_cluster_manager = ConduitClusterManager(_es_client=es_client_, _service_management=_service_management)


@app.route('/')
def index():
    return ' { "status": "ok", "app_version": "1.0" } '


@app.route('/deployment-status', method='GET')
def deployment_status():
    resp = deploy_service_api.get_deployment(conduit_config.get_conduit_license_token())
    if resp:
        data = {
            "status": resp['status'],
            "currentVersion": resp['currentVersion']
        }
        response.headers['Content-Type'] = 'application/json'
        return json.dumps(data)
    else:
        return HTTPResponse(status=404)


@app.route('/versions', method='GET')
def list_available_version():
    logger.debug("getting list of versions")
    try:
        versions = []
        try:
            versions = deploy_service_api.list_versions()
            logger.info("Got %s version(s) from deployment service " % str(len(versions)))
        except exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return HTTPResponse(status=404, headers={'Content-Type': 'application/json'},
                                    body={"error": "No available versions"})
            else:
                logger.exception("Failed to load versions", exc_info=e)
                return HTTPResponse(status=e.response.status_code, headers={'Content-Type': 'application/json'},
                                    body={"error": "Failed to load versions"})

        current_version = None
        try:
            current_version = deploy_service_api.get_current_version(conduit_config.get_conduit_license_token())
            logger.info("Currently registered version %s " % str(current_version))
        except exceptions.HTTPError as e:
            if e.response.status_code == 404:
                return HTTPResponse(status=404, headers={'Content-Type': 'application/json'},
                                    body={"error": "No current version registered"})
            else:
                logger.exception("Failed to get current version", exc_info=e)
                return HTTPResponse(status=e.response.status_code, headers={'Content-Type': 'application/json'},
                                    body={"error": "Failed to load current version"})

        current_version_def = [v for v in versions if v["version"] == current_version]
        available_versions = []
        if len(current_version_def) == 1 and current_version_def[0]['releaseDate'] and current_version_def[0]['releaseDate'] != "":
            logger.info("Filtering final release version list , current release metadata %s " % str(current_version_def[0]))
            current_release_date = dateutil.parser.parse(current_version_def[0]['releaseDate'])
            available_versions = [v for v in versions if v['releaseDate'] and dateutil.parser.parse(v['releaseDate']) > current_release_date]
        else:
            logger.warning("Couldn't resolve current release date from metadata %s " % str(current_version_def))

        response.headers['Content-Type'] = 'application/json'
        logger.info("Available registered versions list : %s " % str(available_versions))
        return json.dumps(available_versions)
    except BaseException as re:
        logger.exception(" deployment-service returned error ", exc_info=re)
        return HTTPResponse(status=500, headers={'Content-Type': 'application/json'},
                            body={"error": "Couldn't load available versions"})


@app.route('/upgrade/<target_version>', method='POST')
def run_update(target_version):
    current_version = deploy_service_api.get_current_version(conduit_config.get_conduit_license_token())
    if not current_version:
        logger.warning("No current version in deployment database")
        return HTTPResponse(status=404, headers={'Content-Type': 'application/json'},
                            body={"error": "No version found"})
    if current_version == target_version:
        logger.info("No migration required, version %s " % current_version)
        return HTTPResponse(status=400, headers={'Content-Type': 'application/json'},
                            body={"error": "Nothing to upgrade, current version is same"})

    target_version_def = deploy_service_api.get_version_def(target_version)
    if not target_version_def:
        return HTTPResponse(status=400, headers={'Content-Type': 'application/json'},
                            body={"error": "version %s not found" % target_version})
    current_status = deploy_service_api.get_deployment_status(conduit_config.get_conduit_license_token())
    if current_status and current_status.is_in_progress():
        return HTTPResponse(status=409, headers={'Content-Type': 'application/json'},
                            body={"error": "upgrade is in progress"})
    payload = request.json or dict()
    dry_run = _to_bool(payload.get("dry_run", request.query.dry_run), False)
    try:
        update_service = UpgradeService(deploy_service_api)
        update_service.execute_update(conduit_config.get_conduit_license_token(), current_version,
                                      target_version, dry_run)
    except UpgradeError as ue:
        logger.exception("Failed to update ", exc_info=ue.cause)
        if not dry_run:
            # set failure only iff it's not a dry run
            set_upgrade_failure_reason(ue.update_context, ue.cause)
            return HTTPResponse(status=500,
                                body={"error": "unexpected error during upgrade. Please contact administrator"})
        else:
            return HTTPResponse(status=500, body={"error": "release verification error"})
    except ConcurrentUpgradeAttempt:
        return HTTPResponse(status=409, headers={'Content-Type': 'application/json'},
                            body={"error": "upgrade is in progress"})
    except ValueError as ve:
        logger.exception("Illegal argument ", exc_info=ve)
        return HTTPResponse(status=400, headers={'Content-Type': 'application/json'},
                            body={"error": str(ve)})
    except BaseException as be:
        logger.exception("Failed to update ", exc_info=be)
        return HTTPResponse(status=500, headers={'Content-Type': 'application/json'},
                            body={"error": str(be)})
    if dry_run:
        return HTTPResponse(status=200)
    else:
        return HTTPResponse(status=201)


@app.route('/install-certificates', method='POST')
def install_certificates():
    current_version = deploy_service_api.get_current_version(conduit_config.get_conduit_license_token())
    if not current_version:
        logger.warning("No current version in deployment database")
        return HTTPResponse(status=404, headers={'Content-Type': 'application/json'},
                            body={"error": "No version found"})
    user_session_json = dict()
    if request.headers['X-AUTH-USER']:
        user_session_json = json.loads(request.headers['X-AUTH-USER'])
    else:
        return HTTPResponse(status=403, headers={'Content-Type': 'application/json'},
                            body={"error": "No user found"})

    conduit_user_email = user_session_json['email']

    try:
        update_service = UpgradeService(deploy_service_api)
        # create update context with same version but with enabled_ssl and fqdn set
        ctx = update_service.create_upgrade_context(
            license_token=conduit_config.get_conduit_license_token(),
            current_version=current_version,
            dry_run=False,
            target_version=current_version,
            extra_vars={"enable_ssl": "true",
                        "conduit_fqdn": conduit_config.get_conduit_fqdn()}
        )

        update_service.install_certs_and_update(conduit_user_email=conduit_user_email,
                                                conduit_fqdn=conduit_config.get_conduit_fqdn(), ctx=ctx)
    except UpgradeError as ue:
        logger.exception("Failed to update ", exc_info=ue.cause)
        set_upgrade_failure_reason(ue.update_context, ue.cause)
        return HTTPResponse(status=500, body={"error": str(ue.cause)})
    except CertificateInstallationError as ce:
        logger.exception("Failed to install SSL certificates  ", exc_info=ce)
        return HTTPResponse(status=500, body={"error": str(ce)})
    except ConcurrentUpgradeAttempt:
        return HTTPResponse(status=409, headers={'Content-Type': 'application/json'},
                            body={"error": "upgrade is in progress"})
    except ValueError as ve:
        logger.exception("Illegal argument ", exc_info=ve)
        return HTTPResponse(status=400, headers={'Content-Type': 'application/json'},
                            body={"error": str(ve)})
    except BaseException as be:
        logger.exception("Failed to update ", exc_info=be)
        return HTTPResponse(status=500, headers={'Content-Type': 'application/json'},
                            body={"error": str(be)})

    return HTTPResponse(status=200)


@app.route('/management/services')
def get_services():
    response.headers['Content-Type'] = 'application/json'
    return json.dumps(_conduit_cluster_manager.get_services_on_local_host())

@app.route('/management/services/zookeeper/status')
def get_services():
    response.headers['Content-Type'] = 'application/json'
    return _conduit_cluster_manager.check_zookeeper_container_state(ServiceId('zookeeper-node'))

@app.route('/management/cluster/services')
def get_cluster_services():
    with_aggregates_services = True

    if "aggregate" in request.query.keys():
        with_aggregates_services = request.query["aggregate"].lower() == "true"

    host = dict()
    vms_hosts = conduit_config.get_deployment_vms()
    for vm in vms_hosts:
        if vm == socket.gethostname():
            host.update(_conduit_cluster_manager.get_services_on_local_host())
        else:
            try:
                host.update(_conduit_cluster_manager.get_services_on_remote_vm(vm))
            except Exception as ex:
                logger.warning("Could not retrieve data for vm {}.".format(vm), exc_info=ex)
                pass

    # If we have more than one VM, we should aggregate services status
    if len(vms_hosts) > 1:
        concatenated_services = []
        for key, value in host.items():
            concatenated_services = concatenated_services + value["services"]
        if with_aggregates_services is None or with_aggregates_services is True:
            host.update({"aggregate_service_health": _conduit_cluster_manager.retrieve_cluster_state(concatenated_services)})
    response.headers['Content-Type'] = 'application/json'
    return json.dumps(host)

@app.route('/management/cluster/services/<service_name>/up')
def get_cluster_services(service_name: str):
    healthy = None
    if service_name == 'kibana' or service_name == 'logstash' or service_name == 'grafana' or service_name == 'graphite':
        healthy = check_all_running(get_service_up_response_from_all_vms(service_name))

    if service_name == 'bde-server' or service_name == 'conduit-master' or service_name == 'conduit-worker' or service_name == 'conduit-proxy' or service_name == 'hive-metastore':
        healthy = check_one_running(get_service_up_response_from_all_vms(service_name))

    if service_name == 'conduit-storage':
        healthy = _conduit_cluster_manager.get_elasticsearch_cluster_health()

    if service_name == 'zookeeper-node':
        healthy = _conduit_cluster_manager.check_zookeeper_cluster_state(_service_management.service_status(service_id=ServiceId(service_name)))

    if healthy == None:
        return HTTPResponse(status=404)
    else:
        if healthy == True:
            return HTTPResponse(status=200)
        else:
            return HTTPResponse(status=500)


def get_service_up_response_from_all_vms(service_name: str):
    responses = []
    vms_hosts = conduit_config.get_deployment_vms()
    for vm in vms_hosts:
        if vm == socket.gethostname():
            response = is_service_up(service_name)
        else:
            response = is_service_up_on_vm(vm, service_name)
        if response.status_code == 404:
            logger.warning("Service %s not found on vm %s", service_name, vm)
            pass
        else:
            logger.info("Service %s has response status code %s from vm %s", service_name, response.status_code, vm)
            responses.append(response)
    return responses


def check_all_running(responses: List[HTTPResponse]):
    for r in responses:
        if r.status_code != 200:
            return False
    return True


def check_one_running(responses: List[HTTPResponse]):
    for r in responses:
        if r.status_code == 200:
            return True
    return False


@app.route('/management/services/<service_name>/status')
def get_service_status(service_name: str):
    try:
        service_status = _service_management.service_status(service_id=ServiceId(service_name))
        response.headers['Content-Type'] = 'application/json'
        return json.dumps(service_status.to_dict())
    except NoSuchService as not_found:
        return HTTPResponse(status=404)


@app.route('/management/services/<service_name>/up')
def is_service_up(service_name: str):
    try:
        service_status = _service_management.service_status(service_id=ServiceId(service_name))
        if service_status.is_running():
            return HTTPResponse(status=200)
        else:
            return HTTPResponse(status=500)
    except NoSuchService as not_found:
        return HTTPResponse(status=404)


@app.route('/management/services/<service_name>/<vm_hostname>/status')
def get_service_status_on_vm(vm_hostname: str, service_name: str):
    try:
        url = "http://{}:{}/management/services/{}/status".format(vm_hostname, server_port, service_name)
        svc_response = requests.get(url)
        return svc_response

    except Exception as ex:
        return HTTPResponse(status=500, body="Could not retrieve services data for {}".format(vm_hostname))


@app.route('/management/services/<service_name>/<vm_hostname>/up')
def is_service_up_on_vm(vm_hostname: str, service_name: str):
    try:
        url = "http://{}:{}/management/services/{}/up".format(vm_hostname, server_port, service_name)
        svc_response_code = requests.get(url).status_code
        return HTTPResponse(status=svc_response_code)

    except Exception as ex:
        return HTTPResponse(status=500, body="Could not retrieve service data for {}".format(vm_hostname))

@app.route('/management/services/<service_name>/<vm_hostname>/restart', method='PUT')
def restart_service_on_vm(service_name: str, vm_hostname: str):
    try:
        url = "http://{}:{}/management/services/{}/restart".format(vm_hostname, server_port, service_name)
        svc_response = requests.put(url)
        return svc_response

    except Exception as ex:
        return HTTPResponse(status=500, body="Could not retrieve service data for {}".format(vm_hostname))


@app.route('/management/services/<service_name>/restart', method='PUT')
def restart_service(service_name: str):

    try:
        service_id = ServiceId(service_name)
        service_status = _service_management.service_status(service_id=service_id)
        if service_status.is_restarting():
            return HTTPResponse(status=200)

        try:
            _service_management.stop_service(service_id=service_id)
        except RuntimeError as stop_error:
            logger.error("failed to stop service %s during restart procedure ".format(str(service_id)),
                         exc_info=stop_error)

        _service_management.start_service(service_id=service_id)
        return HTTPResponse(status=200)

    except NoSuchService as not_found:
        return HTTPResponse(status=404)
    except RuntimeError as re:
        logger.error("Error while ", exc_info=re)
        return HTTPResponse(status=500)


@app.route('/management/services/<service_name>/logs', method='GET')
def service_tail_log(service_name: str):
    follow_log = bool(request.query.follow)
    response.headers['Content-Type'] = 'text/plain'
    return _service_management.service_logs(service_id=ServiceId(name=service_name), follow=follow_log)


@app.route('/backup/repository/local/location', method='GET')
def get_local_backup_location():
    return HTTPResponse(status=200, body=json.dumps({ "location": conduit_config.get_system_snapshots_location()}))


@app.route('/backup/repository', method='POST')
def create_repository():
    body = request.json or dict()
    try:
        validate_response = _backup_service.validate_repository_request(body)
        if validate_response is not None:
            return HTTPResponse(status=400, body={"error": validate_response})
        _backup_service.create_repository_and_policy(body)
    except ConnectionTimeout as es:
        logger.warning("ConnectionTimeout {}".format(str(es)))
        return HTTPResponse(status=408, body={"error": "Connection timeout."})
    except ConnectionError as ce:
        logger.warning("Connection error. {}".format(str(ce)))
        return HTTPResponse(status=ce.status_code, body={"error": "Connection error"})
    except RepositoryVerificationError as rve:
        logger.warning("Repository verification error. {}".format(str(rve)))
        return HTTPResponse(status=rve.status_code, body={"error": "Invalid backup repository configuration. "
                                                                   "Please reconfigure your backup repository."})
    except ElasticsearchException as ee:
        logger.warning("ElasticsearchException. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Elasticsearch exception. "
                                                       "Failed to create repository. Please contact administrator for more info."})
    except errors.APIError as be:
        logger.warning("Docker error. {}".format(str(be)))
        return HTTPResponse(status=be.status_code, body={"error": be.explanation})

    return HTTPResponse(status=200)


@app.route('/backup/repository', method='GET')
def get_repository():
    try:
        es_repo = _backup_service.get_es_repository()
        if es_repo is None:
            return HTTPResponse(status=200, body=json.dumps([]))
        else:
            return HTTPResponse(status=200, body=json.dumps(es_repo))
    except ConnectionTimeout as es:
        logger.warning("ConnectionTimeout {}".format(str(es)))
        return HTTPResponse(status=408, body={"error": "Connection timeout."})
    except ConnectionError as ce:
        logger.warning("Connection error. {}".format(str(ce)))
        return HTTPResponse(status=ce.status_code, body={"error": "Connection error"})
    except RepositoryVerificationError as rve:
        logger.warning("Repository verification error. {}".format(str(rve)))
        return HTTPResponse(status=rve.status_code, body={"error": "Invalid backup repository configuration. "
                                                                   "Please reconfigure your backup repository."})
    except ElasticsearchException as ee:
        logger.warning("ElasticsearchException. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Elasticsearch exception. "
                                                       "Failed to retrieve repository. Please contact administrator for more info."})
    except Exception as ee:
        logger.warning("Repository exception. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Exception. Could not retrieve repository. "
                                                       "Please contact administrator for more info."})


@app.route('/backup/repository/<repo_name>', method='GET')
def get_repository(repo_name: str):
    try:
        es_repo = _backup_service.get_es_repository(repo_name)
        if es_repo is None:
            return HTTPResponse(status=200, body=json.dumps([]))
        else:
            return HTTPResponse(status=200, body=json.dumps(es_repo))
    except ConnectionTimeout as es:
        logger.warning("ConnectionTimeout {}".format(str(es)))
        return HTTPResponse(status=408, body={"error": "Connection timeout."})
    except ConnectionError as ce:
        logger.warning("Connection error. {}".format(str(ce)))
        return HTTPResponse(status=ce.status_code, body={"error": "Connection error"})
    except RepositoryVerificationError as rve:
        logger.warning("Repository verification error. {}".format(str(rve)))
        return HTTPResponse(status=rve.status_code, body={"error": "Invalid backup repository configuration. "
                                                                   "Please reconfigure your backup repository."})
    except ElasticsearchException as ee:
        logger.warning("ElasticsearchException. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Elasticsearch exception. "
                                                       "Failed to retrieve repository. Please contact administrator for more info."})
    except Exception as ee:
        logger.warning("Repository exception. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Exception. Could not retrieve repository. "
                                                       "Please contact administrator for more info."})


@app.route('/backup/snapshot/<repository_name>/<snapshot_name>', method='POST')
def create_snapshot(repository_name, snapshot_name):
    snapshot_settings = {
        "indices": ",".join(conduit_config.get_system_backup_indexes()),
        "ignore_unavailable": True,
        "wait_for_completion": True,
        "include_global_state": False
    }
    try:
        _backup_service.create_snapshot(repository_name, snapshot_name, snapshot_settings)
        return HTTPResponse(status=200)
    except ConnectionTimeout as es:
        logger.warning("ConnectionTimeout {}".format(str(es)))
        return HTTPResponse(status=es.status_code, body={"error": "Connection timeout."})
    except ConnectionError as ce:
        logger.warning("Connection error. {}".format(str(ce)))
        return HTTPResponse(status=ce.status_code, body={"error": "Connection error"})
    except RepositoryVerificationError as rve:
        logger.warning("Repository verification error. {}".format(str(rve)))
        return HTTPResponse(status=rve.status_code, body={"error": "Invalid backup repository configuration. "
                                                                   "Please reconfigure your backup repository."})
    except RepositoryConcurrencyError as rce:
        logger.warning("Repository verification error. {}".format(str(rce)))
        return HTTPResponse(status=rce.status_code, body={"error": "Could not read repository data because the contents of the repository "
                                                                   "do not match its expected state. This is likely the result of either "
                                                                   "concurrently modifying the contents of the repository by a process other than "
                                                                   "this cluster or an issue with the repository’s underlying storage. "
                                                                   "The repository has been disabled to prevent corrupting its contents. "
                                                                   "To re-enable it save backup configuration in order to recreate the repository"})
    except ElasticsearchException as ee:
        logger.warning("ElasticsearchException. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Elasticsearch exception. "
                                                       "Failed to retrieve repository. Please contact administrator for more info."})
    except Exception as ee:
        logger.warning("Repository exception. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Exception. Could not retrieve repository. "
                                                       "Please contact administrator for more info."})


@app.route('/backup/slm/<policy_name>', method='PUT')
def create_lifecycle_management(policy_name):
    snapshot_settings = request.json or dict()
    try:
        _backup_service.create_lifecycle(policy_name, snapshot_settings)
        return HTTPResponse(status=200)
    except ConnectionTimeout as es:
        logger.warning("ConnectionTimeout {}".format(str(es)))
        return HTTPResponse(status=es.status_code, body={"error": "Connection timeout."})
    except ConnectionError as ce:
        logger.warning("Connection error. {}".format(str(ce)))
        return HTTPResponse(status=ce.status_code, body={"error": "Connection error"})
    except ElasticsearchException as ee:
        logger.warning("ElasticsearchException. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Elasticsearch exception. "
                                                       "Failed to retrieve repository. Please contact administrator for more info."})
    except Exception as ee:
        logger.warning("Repository exception. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Exception. Could not retrieve repository. "
                                                       "Please contact administrator for more info."})


@app.route('/backup/slm', method='GET')
def get_slm():
    try:
        es_repo = _backup_service.get_lifecycle("conduit-snapshot")
        if es_repo is None:
            return HTTPResponse(status=200, body=json.dumps(dict()))
        else:
            return HTTPResponse(status=200, body=json.dumps(es_repo))
    except ConnectionTimeout as es:
        logger.warning("ConnectionTimeout {}".format(str(es)))
        return HTTPResponse(status=es.status_code, body={"error": "Connection timeout."})
    except ConnectionError as ce:
        logger.warning("Connection error. {}".format(str(ce)))
        return HTTPResponse(status=ce.status_code, body={"error": "Connection error"})
    except RepositoryVerificationError as rve:
        logger.warning("Repository verification error. {}".format(str(rve)))
        return HTTPResponse(status=rve.status_code, body={"error": "Invalid backup repository configuration."})
    except RepositoryConcurrencyError as rce:
        logger.warning("Repository verification error. {}".format(str(rce)))
        return HTTPResponse(status=rce.status_code, body={"error": "Could not read repository data because the contents of the repository "
                                                                   "do not match its expected state. This is likely the result of either "
                                                                   "concurrently modifying the contents of the repository by a process other than "
                                                                   "this cluster or an issue with the repository’s underlying storage. "
                                                                   "The repository has been disabled to prevent corrupting its contents. "
                                                                   "To re-enable it save backup configuration in order to recreate the repository"})
    except ElasticsearchException as ee:
        logger.warning("ElasticsearchException. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Elasticsearch exception. "
                                                       "Failed to retrieve repository. Please contact administrator for more info."})
    except Exception as ee:
        logger.warning("Repository exception. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Exception. Could not retrieve repository. "
                                                       "Please contact administrator for more info."})


@app.route('/backup/slm/execute-lifecycle/<repository_name>/<policy_name>', method='POST')
def execute_lifecycle(repository_name, policy_name):
    try:
        es_repo = _backup_service.execute_lifecycle(policy_name, repository_name)
        if es_repo is None:
            return HTTPResponse(status=404,
                                body={"error": "Could not execute snapshot on demand. Snapshot policy {} does not exist"
                                      .format(policy_name)})
        else:
            return HTTPResponse(status=200, body={"snapshot_name": es_repo})
    except ConnectionTimeout as es:
        logger.warning("ConnectionTimeout {}".format(str(es)))
        return HTTPResponse(status=es.status_code, body={"error": "Connection timeout."})
    except ConnectionError as ce:
        logger.warning("Connection error. {}".format(str(ce)))
        return HTTPResponse(status=ce.status_code, body={"error": "Connection error"})
    except RepositoryVerificationError as rve:
        logger.warning("Repository verification error. {}".format(str(rve)))
        return HTTPResponse(status=rve.status_code, body={"error": "Invalid backup repository configuration. "
                                                                   "Please reconfigure your backup repository."})
    except RepositoryConcurrencyError as rce:
        logger.warning("Repository verification error. {}".format(str(rce)))
        return HTTPResponse(status=rce.status_code, body={"error": "Could not read repository data because the contents of the repository "
                                                                   "do not match its expected state. This is likely the result of either "
                                                                   "concurrently modifying the contents of the repository by a process other than "
                                                                   "this cluster or an issue with the repository’s underlying storage. "
                                                                   "The repository has been disabled to prevent corrupting its contents. "
                                                                   "To re-enable it save backup configuration in order to recreate the repository"})

    except ElasticsearchException as ee:
        logger.warning("ElasticsearchException. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Elasticsearch exception. "
                                                       "Failed to retrieve repository. Please contact administrator for more info."})
    except Exception as ee:
        logger.warning("Repository exception. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Exception. Could not retrieve repository. "
                                                       "Please contact administrator for more info."})

@app.route('/backup/keystore/<repository_type>', method='POST')
def insert_values_in_es_keystore(repository_type):
    logger.info("Inserting credentials in es keystore for {} repository".format(repository_type))
    json_data = request.json or dict()
    try:
        _backup_service.create_keystore_entries(repository_type, json_data)
    except Exception as e:
        logger.warning("Failed to insert keystore entries. {}".format(str(e)))
        return HTTPResponse(status=500, body={"error": "Failed to insert keystore entries"})

@app.route('/backup/keystore/<repository_type>', method='DELETE')
def insert_values_in_es_keystore(repository_type):
    logger.info("Removing credentials from es keystore for {} repository".format(repository_type))
    try:
        _backup_service.remove_keystore_entries(repository_type)
    except Exception as e:
        logger.warning("Failed to insert keystore entries. {}".format(str(e)))
        return HTTPResponse(status=500, body={"error": "Failed to insert keystore entries"})


@app.route('/backup/snapshots/<snapshot_name>', method='GET')
def get_snapshot(snapshot_name):
    try:
        snapshot_response = _backup_service.get_snapshots(snapshot_name)
        return HTTPResponse(status=200, body=json.dumps(snapshot_response))
    except ConnectionTimeout as es:
        logger.warning("ConnectionTimeout {}".format(str(es)))
        return HTTPResponse(status=es.status_code, body={"error": "Connection timeout."})
    except ConnectionError as ce:
        logger.warning("Connection error. {}".format(str(ce)))
        return HTTPResponse(status=ce.status_code, body={"error": "Connection error"})
    except ElasticsearchException as ee:
        logger.warning("ElasticsearchException. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Elasticsearch exception. "
                                                       "Failed to retrieve repository. Please contact administrator for more info."})
    except Exception as ee:
        logger.warning("Repository exception. {}".format(str(ee)))
        return HTTPResponse(status=500, body={"error": "Exception. Could not retrieve repository. "
                                                       "Please contact administrator for more info."})


@app.route('/vm-info', method='POST')
def get_vm_information():
    json_data = request.json or dict()
    disk_name = None
    measure_unit = None
    vm_info = list()

    if "diskName" in json_data.keys():
        disk_name = json_data["diskName"]
    if "measureUnit" in json_data.keys():
        measure_unit_string = json_data["measureUnit"]
        try:
            if measure_unit_string is not None:
                measure_unit = DigitalInfoUnitOfMeasure[measure_unit_string.upper()]
        except Exception:
            error_message = "Unit of measure {} not supported. Supported values are [MB, GB]".format(measure_unit)
            logger.warning(error_message)
            return HTTPResponse(status=400, body={"error": error_message})

    try:
        vm_info.append(_conduit_cluster_manager.get_vm_status_on_local_vm(disk_name, measure_unit))
        return HTTPResponse(status=200, body=json.dumps(vm_info))
    except DiskNotFoundException as dnf:
        logger.warning("Disk {} not found. {}".format(disk_name, str(dnf)))
        return HTTPResponse(status=dnf.status_code, body={"error": dnf.error})
    except Exception as ee:
        logger.error("Error retrieving vm info.", exc_info=ee)
        return HTTPResponse(status=500, body={"error": "Failed to retrieve info about vm"})


@app.route('/cluster/vm-info', method='POST')
def get_cluster_vm_information():
    json_data = request.json or dict()
    disk_name = None
    measure_unit = None

    if "diskName" in json_data.keys():
        disk_name = json_data["diskName"]
    if "measureUnit" in json_data.keys():
        measure_unit_string = json_data["measureUnit"]
        try:
            if measure_unit_string is not None:
                measure_unit = DigitalInfoUnitOfMeasure[measure_unit_string.upper()]
        except Exception:
            error_message = "Unit of measure {} not supported. Supported values are [MB, GB]".format(measure_unit)
            logger.warning(error_message)
            return HTTPResponse(status=400, body={"error": error_message})

    vm_info = list()
    vms_hosts = conduit_config.get_deployment_vms()

    # get info for singlebox deployment
    if len(vms_hosts) == 1 and vms_hosts[0] == socket.gethostname():
        try:
            vm_info.append(_conduit_cluster_manager.get_vm_status_on_local_vm(disk_name, measure_unit))
        except DiskNotFoundException as dnf:
            logger.warning("Disk {} not found. {}".format(disk_name, str(dnf)))
            return HTTPResponse(status=dnf.status_code, body={"error": dnf.error})
        except Exception as ee:
            logger.error("Error retrieving vm info.", exc_info=ee)
            return HTTPResponse(status=500, body={"error": "Failed to retrieve info about vm"})
    else:
        #get info for ha deployment
        for vm in vms_hosts:
            if vm == socket.gethostname():
                try:
                    vm_info.append(_conduit_cluster_manager.get_vm_status_on_local_vm(disk_name, measure_unit))
                except Exception as ex:
                    logger.warning("Could not retrieve data for vm {}.".format(vm), exc_info=ex)
                    pass
            else:
                try:
                    vm_info = [*vm_info, *_conduit_cluster_manager.get_vm_status_on_remote_vm(vm, json_data)]
                except Exception as ex:
                    logger.warning("Could not retrieve data for vm {}.".format(vm), exc_info=ex)
                    pass
    headers = dict()
    headers['Content-Type'] = "application/json"
    return HTTPResponse(status=200, headers=headers, body=json.dumps(vm_info))


def set_upgrade_failure_reason(upgrade_context: UpgradeContext, error: RuntimeError):
    try:
        logger.exception("Updating failed  deployment status for deploymentToken=%s",
                         conduit_config.get_conduit_license_token(), exc_info=error)

        deploy_service_api.set_deployment_status(upgrade_context, DeploymentStatus.UPGRADE_FAILED)
    except BaseException as be:
        logger.exception("Failed to update deployment status: %s " % str(be), exc_info=be)
        pass


def _to_bool(value, default: bool):
    if value:
        return str(value).lower() == "true"
    else:
        return default


if __name__ == "__main__":
    import __main__ as main
    from ansible.utils.display import Display

    display = Display(verbosity=1)
    setattr(main, 'display', display)

    app.run(host=server_ip, port=server_port, server='paste')
