import os
from collections import namedtuple

from ansible.parsing.dataloader import DataLoader
from ansible.vars.manager import VariableManager
from ansible.inventory.manager import InventoryManager
from ansible.executor.playbook_executor import PlaybookExecutor
from ansible import context
from ansible.module_utils.common.collections import ImmutableDict

from log_config import logger


class AnsibleRunner:

    def __init__(self, _playbook_path: str, inventory_file: str, extra_vars: dict = None, tags: list = list(),
                 check_mode=False):
        """
        Create runner to execute ansible playbook
        :param _playbook_path: location of playbook
        :param extra_vars:
        :param tags:
        :param check_mode: run ansible in check mode if this is True, useful for dry run
        :param inventory_file: pass to inventory file , default is local_hosts file from service configuration
        """
        self._playbook_path = _playbook_path
        self._inventory_file = inventory_file

        if not os.path.exists(self._playbook_path):
            logger.error('[ERROR] The playbook %s does not exist' % self._playbook_path)
            raise FileNotFoundError(path=self._playbook_path)
        if not os.path.exists(self._inventory_file):
            logger.error('[ERROR] The inventory file  %s does not exist' % self._inventory_file)
            raise FileNotFoundError(path=self._inventory_file)

        loader = DataLoader()
        inventory = InventoryManager(loader=loader, sources=self._inventory_file)
        variable_manager = VariableManager(loader=loader, inventory=inventory)

        context.CLIARGS = ImmutableDict(connection='local', module_path=None, forks=100, become=True, listhosts=False, listtags=False,
                                    become_method='sudo', become_user='root', check=check_mode, diff=False, verbosity=4, listtasks=False,
                                    syntax=False, remote_user=None, private_key_file=None, ssh_common_args=None, ssh_extra_args=None,
                                    sftp_extra_args=None, scp_extra_args=None, tags=tags, start_at_task=None)
        local_extra_vars = extra_vars or dict()
        if check_mode:
            local_extra_vars['ansible_check_mode'] = "true"

        for key in local_extra_vars.keys():
            variable_manager.extra_vars[key] = local_extra_vars[key]

        passwords = {}
        logger.info("Prepare playbook %s execution with inventory %s " % (self._playbook_path, self._inventory_file))

        self._pbex = PlaybookExecutor(playbooks=[self._playbook_path], inventory=inventory,
                                      variable_manager=variable_manager, loader=loader,
                                      passwords=passwords)

    def run(self):
        ret_int_code = self._pbex.run()
        logger.info("%s has finished  exit-code=%s , options= %s" % (self._playbook_path, ret_int_code, context.CLIARGS))
        return ret_int_code


if __name__ == "__main__":
    playbook_path = 'debug.yml'
    _extra_vars = dict()
    _extra_vars.update({"bde_version": "latest",  "docker_password": ""})

    import __main__ as main
    from ansible.utils.display import Display
    display = Display(verbosity=4)
    setattr(main, 'display', display)

    _ansible_run = AnsibleRunner(_playbook_path=playbook_path,inventory_file="inventory/local_hosts",
                                 extra_vars=_extra_vars, tags=["start"])
    results = _ansible_run.run()
    print(type(results))
    print(" output: " + str(results))
