import json
import re
import socket
import requests
import conduit_config
from urllib.parse import urljoin

from elasticsearch import Elasticsearch

from conduit_disk_monitoring import DigitalInfoUnitOfMeasure, VMInfo
from service_managment import DockerComposeServiceManager, ServiceStatus, ServiceId
from typing import List
from log_config import logger
from compose.project import NoSuchService


es_client_ = Elasticsearch(hosts=conduit_config.get_elasticsearch_host(), timeout=60)

_service_management = DockerComposeServiceManager(base_dir=conduit_config.get_docker_compose_dir(),
                                                  filenames=conduit_config.get_docker_compose_files())

server_ip = conduit_config.get("service.server.ip")
server_port = int(conduit_config.get("service.server.port"))

class AggregateServiceStatus(object):
    def __init__(self, service_id: ServiceId, is_healthy: bool, display_name: str = None, details: dict = None):
        self.is_healthy = is_healthy
        self.service_id = service_id
        self.display_name = display_name
        self.details = details or dict()

    def to_dict(self):
        return {
            "is_healthy": self.is_healthy,
            "service_id": self.service_id,
            "display_name": self.display_name,
            "details": self.details
        }

class ConduitClusterManager:

    def __init__(self, _es_client: Elasticsearch, _service_management: DockerComposeServiceManager):
        self._service_management = _service_management
        self._es_client = _es_client

    def retrieve_cluster_state(self, concatenated_services: List[ServiceStatus]):
        aggregate_result = []
        for service in concatenated_services:
            service_id = service['service_id']
            res = list(map(lambda x: (x['service_id']), aggregate_result))
            details = dict()
            if len(list(filter(lambda x: (x == service_id), res))) > 0:
                continue
            if service_id == 'kibana' or service_id == 'logstash' or service_id == 'grafana' or service_id == 'graphite':
                is_healthy = self.__check_if_all_services_of_type_are_healthy(service, concatenated_services)
                if not is_healthy:
                    details.update({"message": "At least one {} service is unhealthy".format(service_id)})
                aggregate_result.append(AggregateServiceStatus(service_id, is_healthy, service['display_name'], details).to_dict())

            if service_id == 'bde-server' or service_id == 'conduit-master' or service_id == 'conduit-worker':
                is_healthy = self.__check_if_one_service_of_type_is_healthy(service, concatenated_services)
                if service_id == "spark-worker":
                    is_healthy = self.__check_if_one_service_of_type_is_running(service, concatenated_services)
                if not is_healthy:
                    details.update({"message": "All {} services are unhealthy".format(service_id)})
                aggregate_result.append(AggregateServiceStatus(service_id, is_healthy, service['display_name'], details).to_dict())

            if service_id == 'conduit-storage':
                is_healthy = self.get_elasticsearch_cluster_health()
                if not is_healthy:
                    details.update({"message": "Elasticsearch cluster is unhealthy"})
                aggregate_result.append(AggregateServiceStatus(service_id, is_healthy, service['display_name'], details).to_dict())

            if service_id == 'zookeeper-node':
                is_healthy = self.check_zookeeper_cluster_state(service)
                if not is_healthy:
                    details.update({"message": "Zookeeper cluster is unhealthy"})
                aggregate_result.append(AggregateServiceStatus(service_id, is_healthy, service['display_name'], details).to_dict())

            service_already_exists = len(list(filter(lambda x: (x['service_id'] == service_id), aggregate_result))) > 0
            if service_already_exists is False:
                found_services = list(filter(lambda x: (x['service_id'] == service_id), concatenated_services))
                is_aggregated_status_healthy = len(list(filter(lambda x: (x['is_healthy'] is True), found_services))) > 0
                if not is_aggregated_status_healthy:
                    details.update({"message": "All {} services are unhealthy".format(service_id)})
                aggregate_result.append(AggregateServiceStatus(service_id, is_aggregated_status_healthy,
                                                               service['display_name'], details).to_dict())
        return aggregate_result

    def get_elasticsearch_cluster_health(self):
        cluster_health = self._es_client.cluster.health()
        cluster_status = cluster_health["status"]
        return cluster_status == 'yellow' or cluster_status == 'green'

    def __check_if_all_services_of_type_are_healthy(self, service: ServiceStatus, services: List[ServiceStatus]):
        unhealthy_services = list(filter(lambda x: (x['service_id'] == service['service_id'] and x['is_healthy'] is False), services))
        print("found {} unhealthy services for {}".format(len(unhealthy_services), service['service_id']))
        return len(unhealthy_services) == 0

    def __check_if_one_service_of_type_is_healthy(self, service: ServiceStatus, services: List[ServiceStatus]):
        healthy_services = list(filter(lambda x: (x['service_id'] == service['service_id'] and x['is_healthy'] is True), services))
        return len(healthy_services) > 0

    def __check_if_one_service_of_type_is_running(self, service: ServiceStatus, services: List[ServiceStatus]):
        healthy_services = list(filter(lambda x: (x['service_id'] == service['service_id'] and (x['status'].lower() == 'up' or x['status'].lower() == 'up (healthy)')), services))
        return len(healthy_services) > 0

    def check_zookeeper_cluster_state(self, service: ServiceStatus):
        vms_hosts = conduit_config.get_deployment_vms()
        server_port = int(conduit_config.get("service.server.port"))
        if len(vms_hosts) > 1:
            zk_host_statuses = []
            for vm in vms_hosts:
                url = "http://{}:{}/management/services/zookeeper/status".format(vm, server_port)
                try:
                    svc_response = requests.get(url)
                    if svc_response.status_code == 200:
                        zk_host_statuses.append(svc_response.json())
                except Exception as ex:
                    logger.error("Could not retrieve zookeeper status for {}".format(vm), exc_info=ex)

            for zk_info in zk_host_statuses:
                if "zk_server_state" in zk_info.keys() and zk_info["zk_server_state"] == 'leader':
                    return int(zk_info["zk_synced_follower"]) >= 2
        else:
            # standalone mode
            return service.is_healthy
        return False

    def check_zookeeper_container_state(self, service_id: ServiceId):
        response = dict()
        output = _service_management.exec_cmd(service_id, ["sh", "-c", "echo mntr | nc localhost 2181"])
        output_list = list(output.decode("utf-8").split("\n"))
        zk_info = list(filter(lambda x: ("zk_server_state" in x), output_list))
        if len(zk_info) > 0:
            zk_mode = zk_info[0].split("\t")[1]
            response.update({"zk_server_state": zk_mode})
            if zk_mode == 'leader':
                zk_synced_followers = list(filter(lambda x: ("zk_synced_followers" in x), output_list))
                if len(zk_synced_followers) > 0:
                    zk_synced_follower = zk_synced_followers[0].split("\t")[1]
                    response.update({"zk_synced_follower": zk_synced_follower})
        return response

    def get_services_on_remote_vm(self, vm_hostname: str):
        url = "http://{}:{}/management/services".format(vm_hostname, server_port)
        svc_response = requests.get(url)
        return svc_response.json()

    def get_vm_status_on_remote_vm(self, vm_hostname: str, payload):
        url = "http://{}:{}/vm-info".format(vm_hostname, server_port)
        svc_response = requests.post(url, json=payload)
        return svc_response.json()

    def get_vm_status_on_local_vm(self, disk_name: str, measure_unit: DigitalInfoUnitOfMeasure = None):
        vm_info = VMInfo(disk_name=disk_name, measure_unit=measure_unit).to_dict()

        return vm_info

    def get_services_on_local_host(self):
        hostname = socket.gethostname()
        service_ids = list(self._service_management.services())
        statuses = []
        unhealthy_services = 0
        for service_id in service_ids:
            try:
                service = self._service_management.service_status(service_id=service_id)

                if service_id.name() == "conduit-master":
                    self.__process_conduit_master_status(service)

                if service_id.name() == "conduit-worker":
                    self.__process_conduit_worker_status(service)

                if service_id.name() == "bde-server":
                    self.__process_bde_server_status(service)

                if "up" not in service.status.lower():
                    service.is_healthy = False
                    service.status = "Down"
                if service.is_healthy is False:
                    unhealthy_services += 1
                statuses.append(service.to_dict())
            except NoSuchService as ne:
                logger.warning("Can't find service " + str(service_id), exc_info=ne)
                pass

        return {hostname: {"services": statuses, "unhealthyServices": unhealthy_services}}

    def __process_bde_server_status(self, service: ServiceStatus):
        url = urljoin(conduit_config.get_bde_server_internal_url(), "healthcheck/leader")
        try:
            is_leader_response = requests.get(url)
            if is_leader_response.status_code == 200:
                service.details.update(is_leader_response.json() or dict())
            else:
                service.details.update({"isLeader": False})
                logger.error("Could not retrieve leader status from bde-server")
        except Exception as ex:
            service.details.update({"isLeader": False})
            logger.error("Cannot retrieve leader status from bde-server", exc_info=ex)

    def __process_conduit_master_status(self, service: ServiceStatus):
        service_id = service.service_id
        service.display_name = "Conduit-Master (Spark master)"
        # Check if spark is healthy
        spark_details = self.__get_spark_service_status(service_id, "$SPARK_MASTER_HOST", "$SPARK_MASTER_WEBUI_PORT")
        service.details.update(spark_details)

        service.details.update({"isActive": self.__check_if_spark_master_is_active()})

        # Check if hdfs is used and healthy
        hdfs_details = self.__get_hdfs_service_status_on_spark_container(service_id)
        if hdfs_details['useHdfs']:
            service.display_name = "Conduit-Master (Spark master, HDFS Namenode)"
        service.details.update(hdfs_details)

    def __process_conduit_worker_status(self, service: ServiceStatus):
        service_id = service.service_id
        service.display_name = "Conduit-Worker (Spark worker)"
        # Check if spark is healthy
        spark_details = self.__get_spark_service_status(service_id, "$SPARK_LOCAL_IP", "$SPARK_MASTER_WEBUI_PORT")
        service.details.update(spark_details)

        service.details.update({"isActive": self.__check_if_spark_master_is_active()})

        # Check if hdfs is used and healthy
        hdfs_details = self.__get_hdfs_service_status_on_spark_container(service_id)
        if hdfs_details['useHdfs']:
            service.display_name = "Conduit-Worker (Spark worker, HDFS Namenode)"
        service.details.update(hdfs_details)

    def __get_spark_service_status(self, service_id: ServiceId, host_env_name, port_env_name):
        details = dict()
        try:
            # Check if host_env_name ( SPARK_MASTER_HOST / SPARK_LOCAL_IP ) is defined on container
            spark_host_response = self._service_management.exec_cmd(service_id, ["sh", "-c", "echo {}".format(host_env_name)], demux=True)
            spark_host_stdout = spark_host_response[0]
            spark_host_stderr = spark_host_response[1]
            if spark_host_stderr is not None or spark_host_stdout is None:
                details.update({"isSparkServiceHealthy": False})
                logger.error("Could not check {} env variable on {}. {}".format(host_env_name, service_id, spark_host_stderr))
            else:
                spark_host = spark_host_stdout.decode('utf-8').split("\n")[0]
                if spark_host == "":
                    details.update({"isSparkServiceHealthy": False})
                    logger.error("{} env variable on {} is empty.".format(host_env_name, service_id))
                else:
                    # Check if curl response is 0
                    docker_cmd = ["sh", "-c", "curl --silent --fail -o /dev/null  {}:{}".format(host_env_name, port_env_name)]
                    self._service_management.exec_cmd(service_id, docker_cmd, demux=True)

                    spark_curl_response = self._service_management.exec_cmd(service_id, ["sh", "-c", "echo $?"], demux=True)
                    spark_curl_stdout = spark_curl_response[0]
                    spark_curl_stderr = spark_curl_response[1]

                    if spark_curl_stderr is not None or spark_curl_stdout is None:
                        details.update({"isSparkServiceHealthy": False})
                        logger.error("Could not check spark test response on {}. {}".format(service_id, spark_curl_stderr))
                    else:
                        spark_curl_result = spark_curl_stdout.decode('utf-8').split("\n")[0]
                        details.update({"isSparkServiceHealthy": spark_curl_result == "0"})

        except Exception as ex:
            details.update({"isSparkServiceHealthy": False})
            logger.error("Could not test hdfs connection on {} container".format(service_id), exc_info=ex)
        return details

    def __check_if_spark_master_is_active(self) -> bool:
        # check if spark is active or passive
        try:
            spark_ui_response = requests.get("http://{}:4080".format(server_ip))
            mo = re.compile(r'Status:.+ALIVE').search(spark_ui_response.text)
            return mo is not None
        except Exception as ex:
            logger.error("Could not retrieve spark session state", exc_info=ex)
            return False

    def __get_hdfs_service_status_on_spark_container(self, service_id: ServiceId):
        details = dict()
        # Check if hdfs is used and healthy
        try:
            check_default_fs_response = self._service_management.exec_cmd(service_id, ["sh", "-c", "echo ${FS_DEFAULTFS}"], demux=True)
            default_fs_stdout = check_default_fs_response[0].decode('utf-8').split("\n")[0]
            default_fs_stderr = check_default_fs_response[1]

            if default_fs_stderr is not None:
                logger.warning("Could not check value of FS_DEFAULTFS env variable for {}. {}".format(service_id, str(default_fs_stderr)))
                details.update({"useHdfs": False})
            else:
                if "hdfs://" in default_fs_stdout:
                    details.update({"useHdfs": True})
                    test_result = self._service_management.exec_cmd(
                        service_id, ["sh", "-c", "hdfs dfs -test -e ${FS_DEFAULTFS}/"], demux=True)
                    test_result_stderr = test_result[1]
                    if test_result_stderr is not None:
                        test_error = test_result_stderr.decode('utf-8').split("\n")
                        if len(test_error) > 2:
                            error_message = "; ".join([test_error[0], test_error[1]])
                        else:
                            error_message = "; ".join(test_error)
                        logger.warning("Could not validate hdfs service for {}. {}".format(service_id, error_message))
                        details.update({"isHdfsServiceHealthy": False})
                    else:
                        details.update({"isHdfsServiceHealthy": True})
                else:
                    details.update({"useHdfs": False})
        except Exception as ex:
            details.update({"useHdfs": False})
            logger.error("Could not test hdfs connection on {} container".format(service_id), exc_info=ex)
        return details
