import math
import os
import socket
from typing import List
import multiprocessing

import psutil
from psutil._common import sdiskpart

from enum import Enum


class DigitalInfoUnitOfMeasure(Enum):
    MB = 1
    GB = 2


class DiskNotFoundException(Exception):

    def __init__(self, message):
        self.status_code = 404
        self.error = message


class DiskInfo(object):
    def __init__(self, partition: sdiskpart, measure_unit: DigitalInfoUnitOfMeasure):
        stats = os.statvfs(partition.mountpoint)

        disk_size = stats[1] * stats[2]
        available_size = stats[1] * stats[4]

        size_converted = VMInfo.convert_from_bytes(disk_size, measure_unit)
        available_size_converted = VMInfo.convert_from_bytes(available_size, measure_unit)

        self.disk_size = size_converted
        self.free_space = available_size_converted

        if disk_size > 0:
            self.free_percentage = math.floor((available_size / disk_size) * 100)
        else:
            self.free_percentage = 0

        self.device = partition.device
        self.mountpoint = partition.mountpoint

    def to_dict(self):
        return {
            "diskSize": self.disk_size,
            "freeSpace": self.free_space,
            "freePercentage": self.free_percentage,
            "device": self.device,
            "mountPoint": self.mountpoint
        }

    def __repr__(self):
        to_string = "Device: {}, DiskSize: {}, FreeSpace: {}, FreeSpacePercentage: {}%, MountPoint: {}" \
            .format(self.device, self.disk_size, self.free_space, self.free_percentage, self.mountpoint)
        return "{ " + to_string + " }"


class MemoryInfo(object):
    def __init__(self, measure_unit: DigitalInfoUnitOfMeasure = None):
        memory = psutil.virtual_memory()

        # Show memory in MB
        self.total = VMInfo.convert_from_bytes(memory.total, measure_unit)
        self.available = VMInfo.convert_from_bytes(memory.available, measure_unit)

        self.percentage = memory.percent

    def __repr__(self):
        to_string = "MemoryInfo: [total: {}, available: {}, percentage: {}]"\
            .format(self.total, self.available, str(self.percentage))
        return "{ " + to_string + " }"

    def to_dict(self):
        return {
            "total": self.total,
            "available": self.available,
            "percentage": self.percentage
        }


class VMInfo(object):
    def __init__(self, disk_name: str = None, measure_unit: DigitalInfoUnitOfMeasure = None):
        self.hostname = socket.gethostname()
        if measure_unit is None:
            self.measure_unit = DigitalInfoUnitOfMeasure.MB
        else:
            self.measure_unit = measure_unit
        self.cpu_count = multiprocessing.cpu_count()
        self.disk_info = self.__get_disk_information(disk_name)
        self.memory = MemoryInfo(self.measure_unit)

    @staticmethod
    def convert_from_bytes(bytes_to_convert: int, measure_unit: DigitalInfoUnitOfMeasure = None) -> float:

        if measure_unit is DigitalInfoUnitOfMeasure.MB:
            return round(bytes_to_convert / (1024 * 1024), 1)
        if measure_unit is DigitalInfoUnitOfMeasure.GB:
            return round(bytes_to_convert / (1024 * 1024 * 1024), 1)

        # default conversion is to MB
        return round(bytes_to_convert / (1024 * 1024), 1)

    def __repr__(self):
        to_string = "CPU: {}, Memory: {}, DiskInfo: {}".format(self.cpu_count, self.memory, str(self.disk_info))
        return "{ " + to_string + " }"

    def to_dict(self):
        disk_info_to_dict = list(map(lambda x: x.to_dict(), self.disk_info))
        return {
            "hostname": self.hostname,
            "cpuCount": self.cpu_count,
            "diskInfo": disk_info_to_dict,
            "memory": self.memory.to_dict()
        }

    def __get_disk_information(self, disk_name: str = None) -> List[DiskInfo]:
        partitions = psutil.disk_partitions()
        if disk_name is not None:
            found_partition = list(filter(lambda x: x.device == disk_name, partitions))
            if len(found_partition) == 0:
                raise DiskNotFoundException("Partition '{}' not found".format(disk_name))
            else:
                return list(map(lambda x: DiskInfo(x, self.measure_unit), found_partition))
        else:
            disks_info = map(lambda x: DiskInfo(x, self.measure_unit), partitions)
            return list(filter(lambda x: x.disk_size > 0, list(disks_info)))

