#!/bin/bash

if ! command -v az &>/dev/null; then
  echo "Install Azure cli   https://docs.microsoft.com/en-us/cli/azure/install-azure-cli-apt?view=azure-cli-latest "
  exit 1
fi

if ! command -v envtpl &>/dev/null; then
  echo "Install  envtpl using 'pip install envtpl' "
  exit 2
fi

set -e
#set +x

export RESOURCES_SCRIPT_PATH="$( cd -- "$(dirname "$BASH_SOURCE")"  >/dev/null 2>&1 ; pwd -P )"

# required input parameters
#export DEPLOYMENT_NAME="conduit-ha-dev"
#export SSH_PUBLIC_KEY_FILE=/path/to/ssh.pub
#export DEPLOYMENT_REGION=westus
#export VM_COUNT=3
#export CONDUIT_FS_TYPE=abfs

# varibales
export RESOURCE_GROUP="${DEPLOYMENT_NAME}-resources"
export VNET_NAME="${DEPLOYMENT_NAME}-vnet"
export LB_NAME="${DEPLOYMENT_NAME}-lb"
export NSG_NAME="${DEPLOYMENT_NAME}-ha-nsg"
export BACKEND_SUBNET=${DEPLOYMENT_NAME}-vm-backend

export AZURE_STORAGE_ACCOUNT="$( echo ${DEPLOYMENT_NAME} | sed 's/[_-]//g' )"
export IDENTITY_NAME="${DEPLOYMENT_NAME}-identity"

export VM_TYPE="Standard_D4_v3"
#VM_TYPE="Standard_D4s_v4"
export DEBUG=" -o none "

#TODO: check vCPU usage:
#az vm list-usage -o table --location westus | grep "Standard DSv4 Family vCPUs"

if [[ ! -f "${SSH_PUBLIC_KEY_FILE}" ]]; then
  echo "No public key file found at ${SSH_PUBLIC_KEY_FILE} "
  exit 3
fi

if [[ -z "${DEPLOYMENT_NAME}" ]]; then
  echo "DEPLOYMENT_NAME variable not found"
    exit 4
fi

echo "Resolving public IP address to add into security group"
CLIENT_IP=$(curl http://ifconfig.me)
echo "Public IP address has been resolved as ${CLIENT_IP}"

echo "Creating resource group ${RESOURCE_GROUP} in ${LOCATION}. All resources will be placed into this group"

az group create ${DEBUG} --name ${RESOURCE_GROUP} --location ${DEPLOYMENT_REGION}
echo "Enabling cloud based parquet storage using VMs managed identity ${IDENTITY_NAME} "
az identity create ${DEBUG} --name ${IDENTITY_NAME}  --resource-group ${RESOURCE_GROUP} --location ${DEPLOYMENT_REGION}
echo "Creating storage account ${AZURE_STORAGE_ACCOUNT} "
az storage account create ${DEBUG} --location "$DEPLOYMENT_REGION" --name ${AZURE_STORAGE_ACCOUNT} --resource-group ${RESOURCE_GROUP} --enable-hierarchical-namespace
az storage container create ${DEBUG} --name public --account-name ${AZURE_STORAGE_ACCOUNT} --public-access container
echo "Creating container ${AZURE_STORAGE_ACCOUNT}/parquet-cache-store "
az storage container create ${DEBUG} --name parquet-cache-store --account-name ${AZURE_STORAGE_ACCOUNT}
ACCOUNT_ID=$( az storage account show --name ${AZURE_STORAGE_ACCOUNT} --resource-group ${RESOURCE_GROUP} --query id -o tsv )
PRINCIPAL_ID=$( az identity show --name ${IDENTITY_NAME} --resource-group ${RESOURCE_GROUP} --query principalId -o tsv )
echo "Granting VM identity permissions to access storage account  ${AZURE_STORAGE_ACCOUNT} "
az role assignment create ${DEBUG} --assignee "$PRINCIPAL_ID" --role "Storage Blob Data Contributor"  --scope "${ACCOUNT_ID}"

echo "Creating virtual network and misc azure resources"

az network nsg create ${DEBUG} --resource-group ${RESOURCE_GROUP} --name ${NSG_NAME}
az network vnet create ${DEBUG} --resource-group ${RESOURCE_GROUP}  --name ${VNET_NAME} \
                    --subnet-name ${BACKEND_SUBNET} --address-prefixes 10.1.0.0/16 --subnet-prefixes 10.1.0.0/24


az network nsg rule create ${DEBUG} --resource-group ${RESOURCE_GROUP} --nsg-name ${NSG_NAME} \
  --name ConduitPublicPortRule \
  --priority 1001 \
  --protocol '*' \
  --direction inbound \
  --source-address-prefix '*' \
  --source-port-range '*' \
  --destination-address-prefix '*' \
  --access allow \
  --destination-port-range 80 10002 443

az network nsg rule create  ${DEBUG} --resource-group ${RESOURCE_GROUP} --nsg-name ${NSG_NAME} -n AdminIpRule --priority 2001 \
  --source-address-prefixes ${CLIENT_IP} \
  --destination-address-prefixes '*' --destination-port-ranges 0-65535 --access Allow \
  --protocol Tcp --description "Allow all traffic from Deployer IP address  "


echo "Creating VM availability set"

az vm availability-set create ${DEBUG} -g ${RESOURCE_GROUP}  \
    --name "${DEPLOYMENT_NAME}-availability-set" \
    --platform-fault-domain-count 2 \
    --platform-update-domain-count 2

SSH_USER="azureuser"
echo "Creating VMs based on UbuntuLTS 18.04, with ssh user $SSH_USER and public ssh key $SSH_PUBLIC_KEY_FILE "

for i in $(seq 1 ${VM_COUNT}); do
  VM_NAME="${DEPLOYMENT_NAME}-vm${i}"
  # check if nc exists
  az network nic create ${DEBUG} -g ${RESOURCE_GROUP} \
    --name "${DEPLOYMENT_NAME}-nic${i}" \
    --vnet-name ${VNET_NAME} --subnet ${BACKEND_SUBNET} \
    --network-security-group ${NSG_NAME} \
    --private-ip-address "10.1.0.$((3 + i))"

  az vm create ${DEBUG} -g ${RESOURCE_GROUP} \
    --name "${VM_NAME}" \
    --availability-set "${DEPLOYMENT_NAME}-availability-set" \
    --size ${VM_TYPE} \
    --nics "${DEPLOYMENT_NAME}-nic${i}" \
    --image UbuntuLTS \
     --os-disk-size-gb 64 \
    --admin-username ${SSH_USER} \
    --ssh-key-values "@${SSH_PUBLIC_KEY_FILE}"

  if [[ ! -z "${IDENTITY_NAME}" ]]; then
      az vm identity assign ${DEBUG}  -g  ${RESOURCE_GROUP} --name "${VM_NAME}" --identities  ${IDENTITY_NAME}
  fi
  eval ' export VM${i}_NAME=${VM_NAME} '
  echo "Created VM ${VM_NAME} "
 done

az network public-ip create ${DEBUG} -g ${RESOURCE_GROUP} --name ${LB_NAME}-ip --sku Standard --allocation-method Static --dns-name ${DEPLOYMENT_NAME}
echo "Creating ${LB_NAME}-subnet  subnet for Application Gateway "
az network vnet subnet create ${DEBUG} --name "${LB_NAME}-subnet" --resource-group ${RESOURCE_GROUP} --vnet-name ${VNET_NAME} --address-prefixes 10.1.1.0/24
echo "Creating Application Gateway ${LB_NAME}. It may take up to 30 mins according to Azure Documentation..."
az network application-gateway create ${DEBUG} --name ${LB_NAME} -g ${RESOURCE_GROUP} --capacity 2 --sku Standard_v2 --frontend-port 80 --public-ip-address ${LB_NAME}-ip \
  --vnet-name ${VNET_NAME} --subnet "${LB_NAME}-subnet" --http-settings-port 80 --http-settings-protocol Http --http-settings-cookie-based-affinity Disabled

echo "Configuring load balancer probes "

az network application-gateway probe create ${DEBUG} -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME} -n httpProbe --protocol http --port 80 --path /healthcheck --interval 5 --threshold 3 --timeout 10 --host-name-from-http-settings true
az network application-gateway probe create ${DEBUG} -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME} -n thriftProbe --protocol http --port 10002 --path /cli-status --interval 5 --threshold 3 --timeout 10 --host-name-from-http-settings true
#az network lb probe create -g ${RESOURCE_GROUP} --lb-name ${LB_NAME} -n httpsProbe --protocol tcp --port 443

echo "Configuring load balancer rules for http and thrift endpoints "
# we have to use default AppGateway http settings here appGatewayBackendHttpSettings. As Azure don't allow to delete it as of Aug 2021
az network application-gateway http-settings update ${DEBUG} -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME} --name appGatewayBackendHttpSettings \
  --port 80 --cookie-based-affinity Enabled --enable-probe True --protocol Http --host-name-from-backend-pool True \
  --probe httpProbe

az network application-gateway http-settings create ${DEBUG} -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME} --name ThriftSettings \
  --port 10002 --cookie-based-affinity Enabled --affinity-cookie-name=ConduitApplicationGatewayAffinity \
  --enable-probe True --protocol Http --host-name-from-backend-pool true \
  --probe thriftProbe

#3. listeners
#az network application-gateway http-listener create ${DEBUG}  --frontend-ip DefaultFrontEndIp -g ${RESOURCE_GROUP} \
#                                                    --gateway-name ${LB_NAME} --name HttpListener --frontend-port 80

az network application-gateway frontend-port create ${DEBUG} -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME} --name ThiftFrontEndPort --port 10002

az network application-gateway http-listener create ${DEBUG} --frontend-ip appGatewayFrontendIP -g ${RESOURCE_GROUP} \
  --gateway-name ${LB_NAME} --name ThiftUnsecureListener --frontend-port ThiftFrontEndPort

#az network application-gateway ssl-cert create -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME}  -n LetEncryptFullChain \
#                                --cert-file \path\to\cert\fullchain-exported.pfx --cert-password exportPassw0rd!
#
#az network application-gateway http-listener create --frontend-ip ${LB_NAME}-ip -g ${RESOURCE_GROUP} \
#                                                    --gateway-name ${LB_NAME} --name HttpListener --frontend-port 443 --ssl-cert LetEncryptFullChain
#
#az network application-gateway http-settings update ${DEBUG} -g ${RESOURCE_GROUP}  --gateway-name ${LB_NAME} --name ThriftSettings \
#                                                    --protocol Https --host-name-from-backend-pool true \
#                                                    --root-certs LetEncryptFullChain

#4. address pool
# first two hosts are bde_host
CONDUIT_HOSTS=($(az network nic list -g ${RESOURCE_GROUP} --query "[].ipConfigurations[0].privateIpAddress" -o tsv))

az network application-gateway address-pool create ${DEBUG} -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME} -n ConduitHostsPool --servers "${CONDUIT_HOSTS[@]}"

#5. balancing rules
az network application-gateway rule update ${DEBUG} -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME} --address-pool ConduitHostsPool \
  -n rule1 --http-listener appGatewayHttpListener --rule-type Basic --http-settings appGatewayBackendHttpSettings

az network application-gateway rule create ${DEBUG} -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME} --address-pool ConduitHostsPool \
  -n ThriftRule --http-listener ThiftUnsecureListener --rule-type Basic --http-settings ThriftSettings

#Delete default configuration if any
#az network application-gateway rule delete --gateway-name ${LB_NAME}  -g ${RESOURCE_GROUP} --name rule1 || true
#az network application-gateway http-listener delete --gateway-name ${LB_NAME}  -g ${RESOURCE_GROUP} --name appGatewayHttpListener || true
#az network application-gateway http-settings delete --gateway-name ${LB_NAME} -g ${RESOURCE_GROUP} --name appGatewayBackendHttpSettings || true
#az network application-gateway frontend-ip delete --gateway-name ${LB_NAME} -g ${RESOURCE_GROUP} --name appGatewayFrontendIP || true

# delete default backend pool
az network application-gateway address-pool delete ${DEBUG} -g ${RESOURCE_GROUP} --gateway-name ${LB_NAME} -n appGatewayBackendPool || true

# configure bastion
echo "Creating bastion IP to allow SSH access"
az network public-ip create ${DEBUG} -g ${RESOURCE_GROUP} --name "${DEPLOYMENT_NAME}-bastion-ip" --sku Standard --allocation-method Static

az network nic ip-config update ${DEBUG} -g ${RESOURCE_GROUP} --name ipconfig1 --nic-name "${DEPLOYMENT_NAME}-nic1" --public-ip-address "${DEPLOYMENT_NAME}-bastion-ip"

# configure NAT Gateway for outbound connections
echo "Creating NAT gateway for outbound connections"
az network public-ip create ${DEBUG} -g ${RESOURCE_GROUP} --name "${DEPLOYMENT_NAME}-nat-gateway-ip"  --sku standard  --allocation static

az network nat gateway create  ${DEBUG} -g ${RESOURCE_GROUP}  --name ${DEPLOYMENT_NAME}-nat-gateway \
    --public-ip-addresses "${DEPLOYMENT_NAME}-nat-gateway-ip" \
    --idle-timeout 10

az network vnet subnet update ${DEBUG} -g ${RESOURCE_GROUP}  --vnet-name ${VNET_NAME} --name ${BACKEND_SUBNET} \
    --nat-gateway ${DEPLOYMENT_NAME}-nat-gateway


echo "Created ${VM_COUNT} VMs in resource group ${RESOURCE_GROUP} :"
az vm list -g ${RESOURCE_GROUP} -d --query "[].{name:name, privateIps: privateIps }" -o tsv
