#!/bin/bash

if ! command -v az &>/dev/null; then
  echo "Install Azure cli   https://docs.microsoft.com/en-us/cli/azure/install-azure-cli-apt?view=azure-cli-latest "
  exit 1
fi

if ! command -v envtpl &>/dev/null; then
  echo "Install  envtpl using 'pip install envtpl' "
  exit 2
fi

set -e

RESOURCES_SCRIPT_PATH="$( cd -- "$(dirname "$BASH_SOURCE")" >/dev/null 2>&1 ; pwd -P )"

# required input parameters
# DEPLOYMENT_NAME
# SSH_PUBLIC_KEY_FILE=/path/to/ssh.pub
# DEPLOYMENT_REGION=westus
# VM_COUNT=3
# CONDUIT_FS_TYPE=abfs

# variables
export RESOURCE_GROUP="${DEPLOYMENT_NAME}-resources"
export VNET_NAME="${DEPLOYMENT_NAME}-vnet"
export LB_NAME="${DEPLOYMENT_NAME}-lb"
export NSG_NAME="${DEPLOYMENT_NAME}-ha-nsg"
export BACKEND_SUBNET=${DEPLOYMENT_NAME}-vm-backend

export AZURE_STORAGE_ACCOUNT="$( echo ${DEPLOYMENT_NAME} | sed 's/[_-]//g' )"
export IDENTITY_NAME="${DEPLOYMENT_NAME}-identity"

#export VM_TYPE="Standard_D4_v3"
export VM_TYPE="Standard_D4s_v4"
export DEBUG=" -o none "

#TODO: check vCPU usage:
#az vm list-usage -o table --location westus | grep "Standard DSv4 Family vCPUs"

if [[ ! -f "${SSH_PUBLIC_KEY_FILE}" ]]; then
  echo "No public key file found at ${SSH_PUBLIC_KEY_FILE} "
  exit 3
fi

if [[ -z "${DEPLOYMENT_NAME}" ]]; then
  echo "DEPLOYMENT_NAME variable not found"
    exit 4
fi

echo "Resolving public IP address to add into security group"
CLIENT_IP=$(curl http://ifconfig.me)
echo "Public IP address has been resolved as ${CLIENT_IP}"

echo "Creating resource group ${RESOURCE_GROUP} in ${LOCATION}. All resources will be placed into this group"

az group create ${DEBUG} --name ${RESOURCE_GROUP} --location ${DEPLOYMENT_REGION}

echo "Enabling cloud based parquet storage using VMs managed identity ${IDENTITY_NAME} "
az identity create ${DEBUG} --name ${IDENTITY_NAME}  --resource-group ${RESOURCE_GROUP} --location ${DEPLOYMENT_REGION}
echo "Creating storage account ${AZURE_STORAGE_ACCOUNT} "
az storage account create ${DEBUG} --location "$DEPLOYMENT_REGION" --name ${AZURE_STORAGE_ACCOUNT} --resource-group ${RESOURCE_GROUP} --enable-hierarchical-namespace
az storage container create ${DEBUG} --name public --account-name ${AZURE_STORAGE_ACCOUNT} --public-access container
echo "Creating container ${AZURE_STORAGE_ACCOUNT}/parquet-cache-store "
az storage container create ${DEBUG} --name parquet-cache-store --account-name ${AZURE_STORAGE_ACCOUNT}
ACCOUNT_ID=$( az storage account show --name ${AZURE_STORAGE_ACCOUNT} --resource-group ${RESOURCE_GROUP} --query id -o tsv )
PRINCIPAL_ID=$( az identity show --name ${IDENTITY_NAME} --resource-group ${RESOURCE_GROUP} --query principalId -o tsv )
echo "Granting VM identity permissions to access storage account  ${AZURE_STORAGE_ACCOUNT} "
az role assignment create ${DEBUG} --assignee "$PRINCIPAL_ID" --role "Storage Blob Data Contributor"  --scope "${ACCOUNT_ID}"

echo "Creating virtual network and misc azure resources"

az network nsg create ${DEBUG} --resource-group ${RESOURCE_GROUP} --name ${NSG_NAME}
az network vnet create ${DEBUG} --resource-group ${RESOURCE_GROUP}  --name ${VNET_NAME} \
                    --subnet-name ${BACKEND_SUBNET} --address-prefixes 10.1.0.0/16 --subnet-prefixes 10.1.0.0/24


az network nsg rule create ${DEBUG} --resource-group ${RESOURCE_GROUP} --nsg-name ${NSG_NAME} \
  --name ConduitPublicPortRule \
  --priority 1001 \
  --protocol '*' \
  --direction inbound \
  --source-address-prefix '*' \
  --source-port-range '*' \
  --destination-address-prefix '*' \
  --access allow \
  --destination-port-range 80 10002 443

az network nsg rule create  ${DEBUG} --resource-group ${RESOURCE_GROUP} --nsg-name ${NSG_NAME} -n AdminIpRule --priority 2001 \
  --source-address-prefixes ${CLIENT_IP} \
  --destination-address-prefixes '*' --destination-port-ranges 0-65535 --access Allow \
  --protocol Tcp --description "Allow all traffic from Deployer IP address  "

az network public-ip create ${DEBUG} -g ${RESOURCE_GROUP} --name ${LB_NAME}-ip --sku Standard --allocation-method Static --dns-name ${DEPLOYMENT_NAME}

echo "Creating Azure Load Balancer ${LB_NAME}  "
az network lb create ${DEBUG} -g ${RESOURCE_GROUP} \
    --vnet-name ${VNET_NAME}  \
    --name ${LB_NAME} \
    --frontend-ip-name FrontEndPool \
    --backend-pool-name BackEndPool \
    --public-ip-address ${LB_NAME}-ip \
    --sku Standard

echo "Configuring load balancer probes "
az network lb probe create ${DEBUG} -g ${RESOURCE_GROUP} --lb-name ${LB_NAME} -n httpProbe --protocol http --port 80  --path /healthcheck --interval 5 --threshold 3
az network lb probe create ${DEBUG} -g ${RESOURCE_GROUP} --lb-name ${LB_NAME} -n thriftProbe --protocol http --port 10002 --path /cli-status  --interval 5 --threshold 3
#az network lb probe create -g ${RESOURCE_GROUP} --lb-name ${LB_NAME} -n httpsProbe --protocol tcp --port 443

echo "Configuring load balancer rules for http and thrift endpoints "

az network lb rule create ${DEBUG} -g ${RESOURCE_GROUP} --lb-name ${LB_NAME} \
    --name httpLbRule --protocol tcp --frontend-port 80 --backend-port 80 \
    --frontend-ip-name FrontEndPool  --backend-pool-name BackEndPool \
    --probe-name httpProbe --load-distribution SourceIPProtocol --disable-outbound-snat


az network lb rule create ${DEBUG} -g ${RESOURCE_GROUP} --lb-name ${LB_NAME} \
    --name thriftLbRule --protocol tcp --frontend-port 10002 --backend-port 10002 \
    --frontend-ip-name FrontEndPool  --backend-pool-name BackEndPool \
    --probe-name thriftProbe --load-distribution SourceIPProtocol --disable-outbound-snat

echo "Creating VM availability set"

az vm availability-set create ${DEBUG} -g ${RESOURCE_GROUP}  \
    --name "${DEPLOYMENT_NAME}-availability-set" \
    --platform-fault-domain-count 2 \
    --platform-update-domain-count 2

SSH_USER="azureuser"
echo "Creating VMs based on UbuntuLTS 18.04, with ssh user $SSH_USER and public ssh key $SSH_PUBLIC_KEY_FILE "

for i in $(seq 1 ${VM_COUNT}); do
  VM_NAME="${DEPLOYMENT_NAME}-vm${i}"
  # check if nc exists
  az network nic create ${DEBUG} -g ${RESOURCE_GROUP} \
        --name "${DEPLOYMENT_NAME}-nic${i}" \
        --vnet-name  ${VNET_NAME} --subnet ${BACKEND_SUBNET}  \
        --network-security-group ${NSG_NAME} \
        --private-ip-address "10.1.0.$(( 3 + i))" \
        --lb-name ${LB_NAME} \
        --lb-address-pools BackEndPool

   az vm create ${DEBUG} -g ${RESOURCE_GROUP} \
     --name ${VM_NAME} \
     --availability-set "${DEPLOYMENT_NAME}-availability-set" \
     --size ${VM_TYPE}  \
     --nics "${DEPLOYMENT_NAME}-nic${i}" \
     --image UbuntuLTS \
     --os-disk-size-gb 64 \
     --admin-username ${SSH_USER} \
     --ssh-key-values "@${SSH_PUBLIC_KEY_FILE}"

      if [[ ! -z "${IDENTITY_NAME}" ]]; then
        az vm identity assign ${DEBUG}  -g  ${RESOURCE_GROUP} --name "${VM_NAME}" --identities  ${IDENTITY_NAME}
      fi
      eval ' export VM${i}_NAME=${VM_NAME} '
      echo "Created VM ${VM_NAME} "
 done

echo "Creating NAT rules "

az network lb inbound-nat-rule create ${DEBUG} -g ${RESOURCE_GROUP} --lb-name ${LB_NAME} -n ssh_via_vm1 --protocol Tcp \
    --frontend-port 22 --backend-port 22 --frontend-ip-name FrontEndPool

az network lb outbound-rule create ${DEBUG} -g ${RESOURCE_GROUP} --address-pool BackEndPool --frontend-ip-configs FrontEndPool --idle-timeout 4 --lb-name ${LB_NAME} --name DefaultOutboundRule --protocol All

az network nic ip-config inbound-nat-rule add  ${DEBUG} -g ${RESOURCE_GROUP} --inbound-nat-rule ssh_via_vm1 --ip-config-name ipconfig1 --lb-name ${LB_NAME} --nic-name "${DEPLOYMENT_NAME}-nic1"

echo "Created ${VM_COUNT} VMs in resource group ${RESOURCE_GROUP} :"
az vm list -g ${RESOURCE_GROUP}  -d  --query "[].{name:name, privateIps: privateIps }" -o tsv
