#!/bin/bash

set -e

RESOURCES_SCRIPT_PATH="$( cd -- "$(dirname "$BASH_SOURCE")" >/dev/null 2>&1 ; pwd -P )"
export DEPLOYMENT_TYPE="azure"
LB_7_SCRIPT_PATH=${RESOURCES_SCRIPT_PATH}/create-azure-resources-with-gateway.sh
LB_4_SCRIPT_PATH=${RESOURCES_SCRIPT_PATH}/create-azure-resources.sh
GENERATE_CERT_SCRIPT_PATH=${RESOURCES_SCRIPT_PATH}/generate-letsencrypt-certificate.sh

if [[ ! -f "${RESOURCES_SCRIPT_PATH}/ssh.cfg.tpl"  ]]; then
  echo "No ssh.cfg.tpl found at ${RESOURCES_SCRIPT_PATH} "
  exit 5
fi

if [[ ! -f "${RESOURCES_SCRIPT_PATH}/ha_inventory.tpl"  ]]; then
  echo "No ha_inventory.tpl found at ${RESOURCES_SCRIPT_PATH} "
  exit 6
fi

source ${RESOURCES_SCRIPT_PATH}/../../utils/functions.sh
source ${RESOURCES_SCRIPT_PATH}/../../utils/azure_functions.sh

mkdir -p inventory ~/.conduit/${DEPLOYMENT_NAME}

install_os_requirements "${SCRIPT_PATH}"
validate_os_requirements
validate_azure_os_requirements

parse_generic_conduit_parameters  "$@"

prompt_generic_install_parameters

validate_or_prompt_required_parameters

# duplicate variables here to support retry execution
export RESOURCE_GROUP="${DEPLOYMENT_NAME}-resources"
export LB_NAME="${DEPLOYMENT_NAME}-gateway"
export AZURE_STORAGE_ACCOUNT="$( echo ${DEPLOYMENT_NAME} | sed 's/[_-]//g' )"
DEPLOY_RESOURCE=true
if [[ -f  ~/.conduit/${DEPLOYMENT_NAME}/azure-resources-deployed ]]; then
  echo  "It looks like azure resources have already been successfully deployed for ${DEPLOYMENT_NAME} deployment"
  yes_no_confirmation "Would you like to try recreate resources? Select N to skip this step (Y/N): " DEPLOY_RESOURCE false
fi

if [[ "${DEPLOY_RESOURCE}" == "true" ]]; then
  rm -f ~/.conduit/${DEPLOYMENT_NAME}/azure-resources-deployed
  if [[ ${SESSION_AFFINITY} == "GENERATED_COOKIE" ]]; then
    echo "Cookie-based affinity is selected . Provisioning Application Gateway ( Layer 7 load-balancer ) "
    source ${LB_7_SCRIPT_PATH}
  else
    echo "Provisioning Layer 4 load-balancer "
    source ${LB_4_SCRIPT_PATH}
  fi
  touch ~/.conduit/${DEPLOYMENT_NAME}/azure-resources-deployed
fi
export LB_IP=$(az network public-ip show -g ${RESOURCE_GROUP} --name "${LB_NAME}-ip" --query ipAddress -o tsv)

VMS_ARRAY=( $(az vm list   -g ${RESOURCE_GROUP} --query "[].{name:name}" -o tsv | sort  ) )
for (( i=1; i<=${#VMS_ARRAY[@]}; i++ )); do
  eval ' export VM${i}_NAME=${VMS_ARRAY[(($i-1))]} '
done

echo "Generating SSH configuration ssh.cfg "
envtpl <${RESOURCES_SCRIPT_PATH}/ssh.cfg.tpl > "ssh.cfg"
echo "Generating ansible inventory to inventory/${DEPLOYMENT_NAME}.hosts "
envtpl <${RESOURCES_SCRIPT_PATH}/ha_inventory.tpl > inventory/${DEPLOYMENT_NAME}.hosts

echo "Verifying SSH connectivity to VMs "
ansible --become  --private-key "${SSH_PRIVATE_KEY_FILE}" -i "inventory/${DEPLOYMENT_NAME}.hosts" all -m ping

echo "########################### SSH Access details ######################### "
echo "You can ssh to VM's private IP using your private key and ssh.cfg file that was generated in current directory. "
echo "E.g. ssh -i /path/to/private/key -F ssh.cfg azureuser@10.1.0.4 "
echo "********************************************************************************"
echo "*NOTE*: You also need to enable SSH port in *${NSG_NAME}* network security group"

export JWT_SECRET=$(dd if=/dev/urandom bs=1 count=1024 2>/dev/null | base64  | cut -c -256 | rev)
export DEPLOYMENT_FQDN=$(az network public-ip show -g ${RESOURCE_GROUP} --name ${LB_NAME}-ip --query dnsSettings.fqdn -o tsv)
echo "Starting to deploy Conduit on ${DEPLOYMENT_FQDN} "

CONDUIT_ARGS=" ${CONDUIT_ARGS}  -e transformless_input=yes -e enable_ssl_input=no -e conduit_deployment_auth_token_input=conduit_prod "

if [[ ! -z "${DEPLOYMENT_FQDN}" ]]; then
  CONDUIT_ARGS=" ${CONDUIT_ARGS}  -e conduit_fqdn_input=${DEPLOYMENT_FQDN} "
fi

if [[ ! -z ${CONDUIT_FS_TYPE} ]]; then
  CONDUIT_ARGS="$CONDUIT_ARGS  -e conduit_fs_type=${CONDUIT_FS_TYPE} "
fi

export AZURE_STORAGE_ACCOUNT=$( az storage  account list -g ${RESOURCE_GROUP}  --query "[?name == '${AZURE_STORAGE_ACCOUNT}'].{name:name}" -o tsv )

if [[ ! -z "${AZURE_STORAGE_ACCOUNT}" ]]; then
  CONDUIT_ARGS="${CONDUIT_ARGS}  -e conduit_fs_abfs_storage_account=${AZURE_STORAGE_ACCOUNT}  -e conduit_fs_abfs_container=parquet-cache-store  -e use_keyless_auth=true"
fi

export ANSIBLE_INVENTORY_CACHE=True

ansible-playbook --become \
    --private-key "${SSH_PRIVATE_KEY_FILE}" \
    -i "inventory/${DEPLOYMENT_NAME}.hosts" \
    -e jwt_secret_override="${JWT_SECRET}" \
    -e bde_version="BDE_VERSION_PLACEHOLDER" \
    -e docker_user="conduit-onprem" \
    -e docker_password_input="UPDATE_PASSWORD_HERE" \
    -e conduit_deployment_type="azure"  ${CONDUIT_ARGS}  ./on-premise-provision-singlebox-service.yml

