#!/usr/bin/env bash

RESOURCES_SCRIPT_PATH="$( cd -- "$(dirname "$BASH_SOURCE")" >/dev/null 2>&1 ; pwd -P )"
export DEPLOYMENT_TYPE="gcp"
source ${RESOURCES_SCRIPT_PATH}/../../utils/functions.sh
source ${RESOURCES_SCRIPT_PATH}/../../utils/gcloud_functions.sh

export INSTANCE_MACHINE_TYPE="e2-standard-4"
export JB_INSTANCE_MACHINE_TYPE="e2-micro"

validate_gcloud_os_requirements

install_os_requirements "${RESOURCES_SCRIPT_PATH}"
validate_os_requirements

parse_generic_install_parameters "$@"
parse_generic_conduit_parameters "$@"
load_gcloud_parameters "$@"

load_gcloud_parameters_input

#############################
# Validate input parameters #
#############################
echo "Validating parameters before starting to create resources on Google Cloud Platform"
set_gcloud_resources_names
validate_gcloud_parameters_are_defined
echo "Parameters validation successfully completed"

######################
# Creating resources #
######################
echo "Starting to create resources on Google Cloud Platform"
export OS_USERNAME=`gcloud compute os-login describe-profile --format="value(posixAccounts.username)" --project ${GCP_PROJECT_ID}`
export SERVICE_ACCOUNT="${GCS_IAM_USER}@${GCP_PROJECT_ID}.iam.gserviceaccount.com"
#
# Public IPs
#
echo "Reserve public IP for the load balancer: ${LB_IP}"
gcloud compute addresses create "${LB_IP}" \
    --ip-version=IPV4 \
    --global \
    --project="${GCP_PROJECT_ID}"

export LB_PUBLIC_IP=$(gcloud compute addresses describe "${LB_IP}" --format="get(address)" --global --project "${GCP_PROJECT_ID}")

# Create the jump box
echo "Create public IP for jumpbox VM"
gcloud compute addresses create "${JUMPBOX_IP}" \
    --region="${DEPLOYMENT_REGION}" \
    --project="${GCP_PROJECT_ID}"

export JUMP_BOX_PUBLIC_IP=$(gcloud compute addresses describe "${JUMPBOX_IP}" --format="get(address)" --region "${DEPLOYMENT_REGION}" --project "${GCP_PROJECT_ID}")


#
# network setup
#
echo "Creating private network: ${VNET}"
gcloud compute networks create "${VNET}" \
    --bgp-routing-mode=regional \
    --subnet-mode=custom \
    --project="${GCP_PROJECT_ID}"

echo "Creating private network subnet: ${VNET_SUBNET}"
gcloud compute networks subnets create "${VNET_SUBNET}" \
   --network="${VNET}" \
   --region="${DEPLOYMENT_REGION}" \
   --range 10.1.1.0/24 \
   --project="${GCP_PROJECT_ID}"

echo "Creating regional public static IP address for NAT: ${NAT_IP}"
gcloud compute addresses create "${NAT_IP}" \
    --region="${DEPLOYMENT_REGION}" \
    --project="${GCP_PROJECT_ID}"

export NAT_PUBLIC_IP=$(gcloud compute addresses describe "${NAT_IP}" --format="get(address)" --region "${DEPLOYMENT_REGION}" --project "${GCP_PROJECT_ID}")

echo "Creating regional router: ${ROUTER}"
gcloud compute routers create "${ROUTER}" \
    --network="${VNET}" \
    --region="${DEPLOYMENT_REGION}" \
    --project="${GCP_PROJECT_ID}"

echo "Setting router NAT settings: ${NAT}"
gcloud compute routers nats create "${NAT}" \
    --router="${ROUTER}" \
    --nat-all-subnet-ip-ranges \
    --router-region="${DEPLOYMENT_REGION}" \
    --nat-external-ip-pool="${NAT_IP}" \
    --project="${GCP_PROJECT_ID}"


#
# Google Cloud Storage
#

if [ 'gcs' == ${CONDUIT_FS_TYPE} ]; then
    echo "Using File System on Google Cloud Storage"
    echo "Creating resources needed for 'gcs' file system"
    echo ""

    echo "Creating IAM service account: ${GCS_IAM_USER}"
    gcloud iam service-accounts create "${GCS_IAM_USER}" --display-name="IAM Service Account of Conduit ${DEPLOYMENT_NAME} deployment"

    echo "Creating Cloud storage bucket"
    GCS_BUCKET_NAME="GCS_BUCKET_NAME"
    create_random_gcs_name $GCS_BUCKET_NAME
    echo "generated_name: ${GCS_BUCKET_NAME}"
    echo "Created Cloud storage ${GCS_BUCKET_NAME}"

    echo "Adding Roles to IAM Service account: ${GCS_IAM_USER} on Cloud storage: ${GCS_BUCKET_NAME}"
    gsutil iam ch "serviceAccount:${SERVICE_ACCOUNT}:roles/storage.admin" gs://"${GCS_BUCKET_NAME}"
fi


#
# Virtual Machines setup
#
echo "Creating Virtual Machine template: ${INSTANCE_TEMPLATE}"
if [ 'gcs' == ${CONDUIT_FS_TYPE} ]; then
    gcloud compute instance-templates create "${INSTANCE_TEMPLATE}" \
       --region="${DEPLOYMENT_REGION}" \
       --network="${VNET}" \
       --subnet="${VNET_SUBNET}" \
       --tags=allow-health-check \
       --machine-type="${INSTANCE_MACHINE_TYPE}"  \
       --image-family=ubuntu-1804-lts \
       --image-project=ubuntu-os-cloud \
       --boot-disk-size=80GB \
       --boot-disk-type=pd-balanced \
       --boot-disk-device-name=instance-1 \
       --no-address \
       --service-account="${SERVICE_ACCOUNT}" \
       --scopes="https://www.googleapis.com/auth/cloud-platform" \
       --project="${GCP_PROJECT_ID}"
else
    gcloud compute instance-templates create "${INSTANCE_TEMPLATE}" \
       --region="${DEPLOYMENT_REGION}" \
       --network="${VNET}" \
       --subnet="${VNET_SUBNET}" \
       --tags=allow-health-check \
       --machine-type="${INSTANCE_MACHINE_TYPE}"  \
       --image-family=ubuntu-1804-lts \
       --image-project=ubuntu-os-cloud \
       --boot-disk-size=80GB \
       --boot-disk-type=pd-balanced \
       --boot-disk-device-name=instance-1 \
       --no-address \
       --project="${GCP_PROJECT_ID}"
fi

echo "Creating the jumpbox Virtual Machine"
JUMPBOX_ZONE=$(gcloud compute zones list --filter="name=${DEPLOYMENT_REGION}" --format="get(name)" | awk -F/ '{print $NF}' | head -n 1)
gcloud compute instances create "${JUMPBOX_VM}" \
    --zone="${JUMPBOX_ZONE}" \
    --network="${VNET}" \
    --subnet="${VNET_SUBNET}" \
    --tags=allow-health-check \
    --machine-type="${JB_INSTANCE_MACHINE_TYPE}"  \
    --image-family=ubuntu-1804-lts \
    --image-project=ubuntu-os-cloud \
    --boot-disk-size=10GB \
    --boot-disk-type=pd-balanced \
    --boot-disk-device-name="${JUMPBOX_VM}-boot" \
    --address="${JUMPBOX_IP}" \
    --project="${GCP_PROJECT_ID}"

echo "Starting ${VM_COUNT} VMs from template: ${INSTANCE_TEMPLATE}. Group name: ${INSTANCE_VM_GROUP}"
gcloud compute instance-groups managed create "${INSTANCE_VM_GROUP}" \
   --template="${INSTANCE_TEMPLATE}" \
   --size=${VM_COUNT} \
   --region="${DEPLOYMENT_REGION}" \
   --project="${GCP_PROJECT_ID}"

echo "Waiting until VMs from group: ${INSTANCE_VM_GROUP} are stable"
gcloud compute instance-groups managed wait-until "${INSTANCE_VM_GROUP}" --stable --region "${DEPLOYMENT_REGION}" --project "${GCP_PROJECT_ID}"
MIG_INSTANCES_ARR=($(gcloud compute instance-groups list-instances "${INSTANCE_VM_GROUP}" --region "${DEPLOYMENT_REGION}" --format="value(NAME, ZONE)" --project "${GCP_PROJECT_ID}"))

for ((i=0;i< ${#MIG_INSTANCES_ARR[@]} ;i+=2));
do
    INSTANCE_NAME=${MIG_INSTANCES_ARR[$i]}
    INSTANCE_REGION_ZONE=${MIG_INSTANCES_ARR[$i+1]}

    gcloud compute instances add-metadata "${INSTANCE_NAME}" \
        --zone="${INSTANCE_REGION_ZONE}" \
        --metadata enable-oslogin=TRUE \
        --project="${GCP_PROJECT_ID}"

    #definde internal hostname
    INTERNAL_HOSTNAME="${INSTANCE_NAME}.${INSTANCE_REGION_ZONE}.c.${GCP_PROJECT_ID}.internal"
    INSTANCE_ID=$((i/2 + 1))
    eval ' export VM${INSTANCE_ID}_DOMAIN_NAME=${INTERNAL_HOSTNAME}'

done

echo "Enabling OS Login on jumpbox VM"
gcloud compute instances add-metadata "${JUMPBOX_VM}" \
    --zone="${JUMPBOX_ZONE}" \
    --metadata enable-oslogin=TRUE \
    --project="${GCP_PROJECT_ID}"
echo "Enabled OS Login on jumpbox vm ${JUMPBOX_VM} from zone: ${JUMPBOX_ZONE}"

echo "Adding tag: ${TAG_ALLOW_SSH} to Jumpbox instance: ${JUMPBOX_VM}"
gcloud compute instances add-tags "${JUMPBOX_VM}" \
    --tags="${TAG_ALLOW_SSH}" \
    --zone="${JUMPBOX_ZONE}" \
    --project="${GCP_PROJECT_ID}"

#
# Firewall rules
#
echo "Defining named port on VMs: ${INSTANCE_VM_GROUP}"
gcloud compute instance-groups set-named-ports "${INSTANCE_VM_GROUP}" \
    --named-ports http:80,tcp:5222 \
    --region "${DEPLOYMENT_REGION}" \
    --project="${GCP_PROJECT_ID}"

echo "Create firewall rule to open internal traffic: ${VNET_FIREWALL_ALLOW_INTERNAL}"
gcloud compute firewall-rules create "${VNET_FIREWALL_ALLOW_INTERNAL}" \
    --network "${VNET}" \
    --source-ranges=10.1.1.0/24 \
    --allow all \
    --project="${GCP_PROJECT_ID}"

gcloud compute firewall-rules create "${VNET_FIREWALL_ALLOW_EXTERNAL}" \
    --network "${VNET}" \
    --allow tcp:80,tcp:5222 \
    --project="${GCP_PROJECT_ID}"

echo "Create firewall rule to open allow GCP health check: ${VNET_FIREWALL_HEALTH_CHECK}"
gcloud compute firewall-rules create "${VNET_FIREWALL_HEALTH_CHECK}" \
    --network="${VNET}" \
    --action=allow \
    --direction=ingress \
    --source-ranges=130.211.0.0/22,35.191.0.0/16 \
    --target-tags=allow-health-check \
    --rules=tcp:80,tcp:5222 \
    --project="${GCP_PROJECT_ID}"

#
# Load Balancer
#
echo "Creating health check: ${LB_HEALTH_CHECK}"
gcloud compute health-checks create http "${LB_HEALTH_CHECK}" \
    --port=80 \
    --check-interval="5s" \
    --timeout="5s" \
    --healthy-threshold=2 \
    --unhealthy-threshold=2 \
    --request-path="/healthcheck" \
    --project="${GCP_PROJECT_ID}"

gcloud compute health-checks create http "${LB_HEALTH_CHECK_SSL}" \
    --port=5222 \
    --check-interval="5s" \
    --timeout="5s" \
    --healthy-threshold=2 \
    --unhealthy-threshold=2 \
    --request-path="/cli-status" \
    --project="${GCP_PROJECT_ID}"

echo "Creating the load balancer service: ${LB_BACKEND_SERVICE}"
# session-affinity=[NONE/CLIENT_IP/GENERATED_COOKIE]
gcloud compute backend-services create "${LB_BACKEND_SERVICE}" \
    --protocol=HTTP \
    --port-name=http \
    --health-checks="${LB_HEALTH_CHECK}" \
    --session-affinity="${SESSION_AFFINITY}" \
    --global \
    --project="${GCP_PROJECT_ID}"

SESSION_AFFINITY_SSL="${SESSION_AFFINITY}"
if [ 'GENERATED_COOKIE' == "${SESSION_AFFINITY}" ]; then
    echo "Defaulting SSL Load Balancer Session Affinity (for Thrift Port) to CLIENT_IP"
    export SESSION_AFFINITY_SSL="CLIENT_IP"
fi


gcloud compute backend-services create "${LB_BACKEND_SERVICE_SSL}" \
    --protocol=TCP \
    --port-name=tcp \
    --health-checks="${LB_HEALTH_CHECK_SSL}" \
    --session-affinity="${SESSION_AFFINITY_SSL}" \
    --timeout="57600s" \
    --global \
    --project="${GCP_PROJECT_ID}"

echo "Assigning the VM group instances: ${INSTANCE_VM_GROUP} to the backend service: ${LB_BACKEND_SERVICE} "
gcloud compute backend-services add-backend "${LB_BACKEND_SERVICE}" \
    --instance-group="${INSTANCE_VM_GROUP}" \
    --instance-group-region="${DEPLOYMENT_REGION}" \
    --balancing-mode="RATE" \
    --max-rate-per-instance=2000 \
    --global \
    --project="${GCP_PROJECT_ID}"

gcloud compute backend-services add-backend "${LB_BACKEND_SERVICE_SSL}" \
    --instance-group="${INSTANCE_VM_GROUP}" \
    --instance-group-region="${DEPLOYMENT_REGION}" \
    --balancing-mode="CONNECTION" \
    --max-connections-per-instance=2000 \
    --global \
    --project="${GCP_PROJECT_ID}"

#
# SSL Certificate registration
#
if [ -z ${SSL_CERTIFICATE_NAME} ]; then
    export SSL_CERTIFICATE_NAME="${DEPLOYMENT_NAME}-ssl-crt"
    echo "Registering new SSL certificate as: ${SSL_CERTIFICATE_NAME}"
    gcloud compute ssl-certificates create "${SSL_CERTIFICATE_NAME}" \
        --certificate="${SSL_CERTIFICATE_PUBLIC_FILE}" \
        --private-key="${SSL_CERTIFICATE_PRIVATE_FILE}" \
        --project="${GCP_PROJECT_ID}"
fi

#
# HTTPS protocol mapping
#
echo "Creating URL maps: ${LB_URL_MAPS_HTTPS} for backend services: ${LB_BACKEND_SERVICE}"
gcloud compute url-maps create "${LB_URL_MAPS_HTTPS}" \
    --default-service "${LB_BACKEND_SERVICE}" \
    --project="${GCP_PROJECT_ID}"

echo "Creating HTTPS proxy: ${LB_TARGET_PROXY_HTTPS} on the URL map: ${LB_URL_MAPS_HTTPS} using SSL certificate: ${SSL_CERTIFICATE_NAME}"
gcloud compute target-https-proxies create "${LB_TARGET_PROXY_HTTPS}" \
    --url-map="${LB_URL_MAPS_HTTPS}" \
    --ssl-certificates="${SSL_CERTIFICATE_NAME}" \
    --project="${GCP_PROJECT_ID}"

echo "Creating HTTPS (443) forwarding rule from IP service: ${LB_IP} to proxy: ${LB_TARGET_PROXY_HTTPS}"
gcloud compute forwarding-rules create "${LB_FORWARD_RULE_HTTPS}" \
    --address="${LB_IP}" \
    --global \
    --target-https-proxy="${LB_TARGET_PROXY_HTTPS}" \
    --ports=443 \
    --project="${GCP_PROJECT_ID}"

#
# HTTP protocol mapping and redirect to HTTPS
#

echo "Preparing the redirect mapping file to use ${DEPLOYMENT_FQDN}"
envtpl <${RESOURCES_SCRIPT_PATH}/web-map-http.yaml.tpl > ${RESOURCES_SCRIPT_PATH}/web-map-http.yaml

echo "Validating the http->https redirect mapping"
gcloud compute url-maps validate --source "${RESOURCES_SCRIPT_PATH}/web-map-http.yaml"

echo "Creating the http->https redirect mapping"
gcloud compute url-maps import "${LB_URL_MAPS_HTTP}" --source "${RESOURCES_SCRIPT_PATH}/web-map-http.yaml" --global

echo "Creating HTTP proxy: ${LB_TARGET_PROXY_HTTP} on the URL map: ${LB_URL_MAPS_HTTP}"
gcloud compute target-http-proxies create "${LB_TARGET_PROXY_HTTP}" \
   --url-map="${LB_URL_MAPS_HTTP}" \
   --global \
   --project="${GCP_PROJECT_ID}"

echo "Creating HTTP (80) forwarding rule from IP service: ${LB_IP} to proxy: ${LB_TARGET_PROXY_HTTP}"
gcloud compute forwarding-rules create "${LB_FORWARD_RULE_HTTP}" \
    --address="${LB_IP}" \
    --global \
    --target-http-proxy="${LB_TARGET_PROXY_HTTP}" \
    --ports=80 \
    --project="${GCP_PROJECT_ID}"


#
# Thrift server ssl proxy (port 5222 instead of 10002)
#
# https://cloud.google.com/load-balancing/docs/ssl-certificates/troubleshooting

gcloud compute target-ssl-proxies create "${LB_TARGET_PROXY_SSL}" \
   --backend-service="${LB_BACKEND_SERVICE_SSL}" \
   --ssl-certificates="${SSL_CERTIFICATE_NAME}" \
   --project="${GCP_PROJECT_ID}"

gcloud compute forwarding-rules create "${LB_FORWARD_RULE_SSL}" \
    --address="${LB_IP}" \
    --global \
    --target-ssl-proxy="${LB_TARGET_PROXY_SSL}" \
    --ports=5222 \
    --project="${GCP_PROJECT_ID}"

#
# SSL Certificate
#
echo "Adding SSL certificate: ${SSL_CERTIFICATE_NAME} to the Load Balancer proxy: ${LB_TARGET_PROXY_HTTPS}"
gcloud compute target-https-proxies update "${LB_TARGET_PROXY_HTTPS}" \
    --ssl-certificates "${SSL_CERTIFICATE_NAME}" \
    --global-ssl-certificates \
    --global \
    --project="${GCP_PROJECT_ID}"

echo "Adding SSL certificate: ${SSL_CERTIFICATE_NAME} to the Load Balancer proxy: ${LB_TARGET_PROXY_SSL}"
gcloud compute target-ssl-proxies update "${LB_TARGET_PROXY_SSL}" \
    --ssl-certificates "${SSL_CERTIFICATE_NAME}" \
    --project="${GCP_PROJECT_ID}"

#
# SSH Config
#
echo "Generating PUBLIC key for Conduit VMs using selected private key ${SSH_PRIVATE_KEY_FILE}"
    ssh-keygen -y -f "${SSH_PRIVATE_KEY_FILE}" > "${RESOURCES_SCRIPT_PATH}/conduit_id_rsa.pub"
    export SSH_PUBLIC_KEY_FILE="${RESOURCES_SCRIPT_PATH}/conduit_id_rsa.pub"
echo "Adding SSH key from path: ${SSH_PUBLIC_KEY_FILE} as OS Login enabled on VM instances"
gcloud compute os-login ssh-keys add \
    --key-file "${SSH_PUBLIC_KEY_FILE}" \
    --project="${GCP_PROJECT_ID}"

echo "Creating firewall rule to allow SSH"
gcloud compute firewall-rules create "${VNET_FIREWALL_ALLOW_SSH}" \
    --network="${VNET}" \
    --action=allow \
    --direction=ingress \
    --target-tags="${TAG_ALLOW_SSH}" \
    --rules=tcp:22 \
    --priority=100 \
    --project="${GCP_PROJECT_ID}"


#
# Env config
#
echo "Preparing ssh.cfg and ansible inventory file: ${DEPLOYMENT_NAME}.hosts "
mkdir -p ${RESOURCES_SCRIPT_PATH}/inventory
envtpl <${RESOURCES_SCRIPT_PATH}/ssh.cfg.tpl > ${RESOURCES_SCRIPT_PATH}/../../ssh.cfg
envtpl <${RESOURCES_SCRIPT_PATH}/ha_inventory.tpl > ${RESOURCES_SCRIPT_PATH}/../../inventory/${DEPLOYMENT_NAME}.hosts

echo "The GCP infrastructure is ready"
echo "The load balancer public is: ${LB_PUBLIC_IP}"

###################
### Ansible run ###
###################

echo "Starting to deploy The Conduit"
export JWT_SECRET=$(dd if=/dev/urandom bs=1 count=1024 2>/dev/null | base64  | cut -c -256 | rev)

ansible-playbook -vv --become \
    --private-key "${SSH_PRIVATE_KEY_FILE}" \
    -i "${RESOURCES_SCRIPT_PATH}/../../inventory/${DEPLOYMENT_NAME}.hosts" \
    -u "${OS_USERNAME}"  \
    -e transformless_input=yes \
    -e bde_version="${BDE_VERSION}" \
    -e conduit_fs_type="${CONDUIT_FS_TYPE}" \
    -e docker_user="${DOCKER_USER}" \
    -e docker_password_input="${DOCKER_PASSWORD}" \
    -e enable_ssl_input=no \
    -e enable_swagger_https=true \
    -e conduit_fqdn_input="${DEPLOYMENT_FQDN}" \
    -e conduit_admin_name_input="${CONDUIT_ADMIN_FIRSTNAME}" \
    -e conduit_admin_surname_input="${CONDUIT_ADMIN_LASTNAME}" \
    -e conduit_admin_email_input="${CONDUIT_ADMIN_EMAIL}" \
    -e conduit_admin_password_input="${CONDUIT_ADMIN_PASSWORD}" \
    -e jwt_secret_override="${JWT_SECRET}" \
    -e conduit_deployment_type="gcp" \
    -e bde_hive_thrift_http_port_override=5222 \
    -e conduit_deployment_auth_token_input="${CONDUIT_DEPLOYMENT_SERVICE_TOKEN}" \
    -e conduit_fs_gcs_bucket="${GCS_BUCKET_NAME}" \
    -e conduit_fs_gcs_project_id="${GCP_PROJECT_ID}" \
    -e conduit_fs_gcs_service_account_keyfile="" \
    -e conduit_fs_aws_bucket="${AWS_S3_BUCKET_NAME}" \
    -e conduit_fs_aws_access_key="${AWS_S3_BUCKET_ACCESS_KEY}" \
    -e conduit_fs_aws_secret_key="${AWS_S3_BUCKET_SECRET_KEY}" \
    -e conduit_fs_abfs_storage_account="${AZURE_STORAGE_NAME}" \
    -e conduit_fs_abfs_container="${AZURE_CONTAINER_NAME}" \
    -e conduit_fs_abfs_access_key="${AZURE_CONTAINER_ACCESS_KEY}" \
    ${RESOURCES_SCRIPT_PATH}/../../on-premise-provision-singlebox-service.yml


echo ""
echo "Conduit deployment completed"
echo "################################################"
echo "Load balancer public IP: ${LB_PUBLIC_IP}"
echo "Virtual machines outbound public IP: ${NAT_PUBLIC_IP}"
echo "Management console available at: https://${DEPLOYMENT_FQDN}"
echo "Please use account '${CONDUIT_ADMIN_EMAIL}' to login"
echo "################################################"
echo ""
