#!/usr/bin/env bash

SUDO_ACCESS=$(id -u)
if [ "$SUDO_ACCESS" -ne 0 ]; then
  echo "Root permission required to execute Conduit restore"
  exit 2
fi
INVENTORY_FILE=inventory/${1:-"local_hosts"}

if [ ! -f "$INVENTORY_FILE" ]; then
    echo "Inventory file $INVENTORY_FILE does not exist."
    exit 2
fi

IS_LOCAL_ENVIRONMENT=${2:-"false"}
shopt -s nocasematch

  case "$IS_LOCAL_ENVIRONMENT" in
    yes)
      IS_LOCAL_ENVIRONMENT=true ;;
    true)
      IS_LOCAL_ENVIRONMENT=true ;;
    *)
      IS_LOCAL_ENVIRONMENT=false ;;
  esac

DISTRIBUTION=$(uname -s)
LINUX_DISTRIBUTION="Linux"
PROVISIONING_TOKEN=""
REPOSITORY_TYPE=""

if [[ "$DISTRIBUTION" == "$LINUX_DISTRIBUTION" ]] ; then
  if ! command -v ansible &> /dev/null
  then
    echo "Installing ansible "
    sudo apt-get install -y software-properties-common whois jq at \
                       python3 python3-pip python3-apt python3-setuptools
    sudo python3 -m pip install --upgrade pip
    sudo python3 -m pip install ansible==2.9.13 requests

  fi
fi

execute_restore() {
    repo_name="${1:-conduit_snapshot_repository}"

    echo "Repository created. We'll soon display available snapshots names."
    ansible-playbook -i "$INVENTORY_FILE" conduit-restore-get-snapshots.yml -e repository_name="$repo_name"
    get_snaphsots_response=$?
    if [ $get_snaphsots_response -ne 0 ]; then
        echo "Restore could not be done. Please contact administrator"
        exit 2
    fi

    ansible-playbook ./conduit-restore.yml --become -e repository_name="$repo_name" -i "$INVENTORY_FILE" -e "repository_type=$REPOSITORY_TYPE"  -e "provisioning_token_input=$PROVISIONING_TOKEN"  -e "is_local_environment=${IS_LOCAL_ENVIRONMENT}"
    ansible_response=$?
    if [ $ansible_response -ne 0 ]; then
        echo "Restore could not be done. Please contact administrator"
        exit 2
    else
      echo "Restore was successful"
      exit 0
    fi
}


verify_ssh_connection() {
  ansible-playbook -i "$INVENTORY_FILE" test_ansible_ssh_connection.yml
  SSH_CONNECTION_TEST=$?

  if [ $SSH_CONNECTION_TEST -eq 0 ]; then
    echo "SSH connection successfully tested"
  else
    if [ $SSH_CONNECTION_TEST -eq 4 ]; then
      echo "SSH connection unreachable"
    else
      echo "Unable to test ssh connection"
    fi

    echo "Please enter path to private key that should be used by vm to execute restore (leave empty for single VM deployments)"
    read -r ssh_key

    if [ ! -f "$ssh_key" ] && [ ! -z "$ssh_key" ];
    then
      echo "File does not exist, please enter correct ssh key path"
      read -r ssh_key
      if [ ! -f "$ssh_key" ] && [ ! -z "$ssh_key" ];
      then
        echo "Could not find ssh key file, please add ssh file and try again"
        exit 2
      fi
    fi

    if [ ! -z "$ssh_key" ];
    then
      eval `ssh-agent` > /dev/null
      echo "adding $ssh_key to ssh-agent"
      ssh-add "$ssh_key"
    fi


    ansible-playbook -i "$INVENTORY_FILE" test_ansible_ssh_connection.yml
    SSH_CONNECTION_TEST=$?

    if [ $SSH_CONNECTION_TEST -ne 0 ]; then
      echo "Restore could not be done. Cannot connect to hosts based on SSH key. Please contact administrator"
      exit 2
    fi
  fi


}

create_repository() {
  read_only="${1:-false}"
  repo_name="${2:-conduit_snapshot_repository}"

  echo "Please enter repository type (fs / azure / s3 / gcs)"
  read -r repository_type_input
  repository_type=$(echo "$repository_type_input" | tr '[:upper:]' '[:lower:]')
  REPOSITORY_TYPE=$repository_type
  case $repository_type in

    s3)
      ANSIBLE_HOST_KEY_CHECKING=False ansible-playbook -i "$INVENTORY_FILE" ./conduit-restore-s3-repository.yml -e "read_only=$read_only" -e "repository_name=$repo_name"
      ;;

    azure)
      ANSIBLE_HOST_KEY_CHECKING=False ansible-playbook -i "$INVENTORY_FILE" ./conduit-restore-azure-repository.yml -e "read_only=$read_only" -e "repository_name=$repo_name"
      ;;

    gcs)
      ANSIBLE_HOST_KEY_CHECKING=False ansible-playbook -i "$INVENTORY_FILE" ./conduit-restore-gcs-repository.yml -e "read_only=$read_only" -e "repository_name=$repo_name"
      ;;
    fs)
      ansible-playbook -i "$INVENTORY_FILE" ./conduit-restore-fs-repository.yml -e "read_only=$read_only" -e "repository_name=$repo_name"
      ;;

    *)
      echo "Unknown value. Please start the script again and enter one of allowed repositories (fs / azure / s3 / gcs)"
      exit 2
  esac
  ansible_response=$?
  if [ $ansible_response -ne 0 ]; then
      echo "Failed create repository in elasticsearch"
      exit 2
  fi

}

verify_existing_repository() {
  repo_name="${1:-"conduit_snapshot_repository"}"
  ansible-playbook -i "$INVENTORY_FILE" ./conduit-restore-verify-repository.yml -e "repository_name=$repo_name" -e "scenario=missing_repository"
  ansible_response=$?
  if [ "$ansible_response" -ne 0 ]; then
    eval "${2}=1"
    return
  else
      ansible-playbook -i "$INVENTORY_FILE" ./conduit-restore-verify-repository.yml -e "repository_name=$repo_name" -e "scenario=valid_repository"
      ansible_response=$?
      if [ "$ansible_response" -eq 0 ]; then
        eval "${2}=0"
      else
        echo "Elasticsearch could not verify repository. Please contact administrator"
        exit 2
      fi
  fi
}

create_missing_repository() {

    echo "If restore snapshot was created on another environment, PROVISIONING_TOKEN from that environment is required. Otherwise leave blank and press Enter"
    read -r PROVISIONING_TOKEN

    create_repository true conduit_restore_repository
    CONDUIT_CREATE_BACKUP_REPOSITORY=$?

    if [ $CONDUIT_CREATE_BACKUP_REPOSITORY -eq 0 ]; then
      execute_restore conduit_restore_repository
    else
      echo Repository could not be created. Please contact administrator
      exit 2
    fi
}

### Script entrypoint ###

verify_ssh_connection
ansible-playbook -i "$INVENTORY_FILE" ./conduit-restore-start-es-service.yml -e "is_local_environment=${IS_LOCAL_ENVIRONMENT}"
verify_existing_repository "conduit_snapshot_repository" VERIFY_REPOSITORY_CODE

if [ "$VERIFY_REPOSITORY_CODE" -eq 0 ]; then
  echo "Do you want to restore from current configured repository or to create a new one ?"
  echo "'YES' to use current configured repository or 'NO' to use another repository"
  read -r create_repository_response_input
  create_repository_response=$(echo "$create_repository_response_input" | tr '[:upper:]' '[:lower:]')
  case $create_repository_response in

  yes)
    if [ "$VERIFY_REPOSITORY_CODE" -eq 0 ]; then
      execute_restore conduit_snapshot_repository
    fi
    ;;
  no)
    echo "Please enter details to create a repository"
    create_missing_repository
    ;;
  *)
    echo "Unknown value. Please start the script again and enter one of allowed responses (yes / no)"
    exit 2
esac
else
  echo "Repository missing, please enter details to recreate repository"
  create_missing_repository
fi

