#!/usr/bin/env bash

SCRIPT_PATH="$( cd -- "$(dirname "$BASH_SOURCE")" >/dev/null 2>&1 ; pwd -P )"
source ${SCRIPT_PATH}/functions.sh

function validate_azure_os_requirements {
    if ! command -v az &>/dev/null; then
      echo "Install Azure cli   https://docs.microsoft.com/en-us/cli/azure/install-azure-cli-apt?view=azure-cli-latest "
      exit 1
    fi
}


function select_subscription() {
  MSG=$1
  OUTPUT_VAR_NAME=$2
  DEFAULT_ANSWER="$3"

  if [[ $(az account list --output tsv | wc -l) -gt "1" ]]; then
    echo "Multiple subscriptions found"
    az account list --output table
    echo "--------------------------------------------------"
    echo "Current subscription: "
    az account list --output table | grep "${SUBSCRIPTION_ID}"
    yes_no_confirmation $MSG reply_yn
    if [[ "${reply_yn}" == false ]]; then
      prompt_non_empty_str_value "Please provide the desired SubscriptionId, from the list displayed above. " ${OUTPUT_VAR_NAME} ${DEFAULT_ANSWER}
    fi
  fi

}


function validate_or_prompt_required_parameters {

    if [ -z ${DEPLOYMENT_NAME} ]; then
        prompt_non_empty_str_value "Enter the deployment name: " DEPLOYMENT_NAME
        export DEPLOYMENT_NAME
    fi
    echo "Validating deployment name"
    NAMED_DEPLOYMENT_NAME=`az group list --query "[?name=='${DEPLOYMENT_NAME}-resources'].name" -o tsv`
    if [ ! -z ${NAMED_DEPLOYMENT_NAME} ]; then
       echo "The deployment: ${DEPLOYMENT_NAME} already exists. Please Choose a different name. Exiting now"
       yes_no_confirmation "Would you like to try overwrite this deployment ? (Y/N): " REDEPLOY false
       if [[ "${REDEPLOY}" == "false" ]]; then
         echo "Terminating deployment procedure... "
         exit 1
       fi
    fi

    if [ -z ${DEPLOYMENT_REGION} ]; then
        #AVAILABLE_REGIONS=`az account list-locations --query [].name -o table`
        prompt_non_empty_str_value "Enter the Azure location for this deployment: " DEPLOYMENT_REGION
        export DEPLOYMENT_REGION
    fi

    if [[ -z "${VM_COUNT}" ]]; then
        prompt_str_value "Enter the number of instances to deploy [minimal default 3]: " VM_COUNT 3
        export VM_COUNT
    fi

    echo "Validating number of virtual machines"
    if [[ $VM_COUNT =~ '!^[0-9]+$' || "$VM_COUNT" -lt "3" ]] ; then
       echo "Invalid input for the number of virtual machines: ${VM_COUNT}. Exiting now"
       exit 1
    fi

    if [ -z ${CONDUIT_FS_TYPE} ]; then
        echo "Conduit uses  filesystem cache to improve performance of certain types of queries. It's possible to use cloud-based storage for this purpose"
        echo "We recommend to use cloud specific storage, ABFS for Azure, GCS for Google Cloud or S3 for AWS deployment"
        echo "Deployment script will create a dedicated cloud storage to configure Conduit filesystem cache."
        PS3='Choose filesystem type for Conduit Parquet Store: '
        options=("abfs" "hdfs" "s3" "gcs")
        select opt in "${options[@]}"
        do
            case $opt in
                "abfs")
                    CONDUIT_FS_TYPE="abfs"
                    break
                    ;;
                "hdfs")
                    CONDUIT_FS_TYPE="hdfs"
                    break
                    ;;
                "s3")
                    CONDUIT_FS_TYPE="s3"
                    break
                    ;;
                "gcs")
                    CONDUIT_FS_TYPE="gcs"
                    break
                    ;;
                *)
                    echo "invalid option $REPLY, using default abfs  "
                    CONDUIT_FS_TYPE="abfs"
                    ;;
            esac
        done

        export CONDUIT_FS_TYPE
    fi

    echo "Validating conduit file system type"
    if [[ ! $CONDUIT_FS_TYPE =~ ^(hdfs|abfs|s3|gcs)$ ]]; then
        echo "Invalid File System type: ${CONDUIT_FS_TYPE}, expected one of [hdfs|abfs|s3|gcs]. Exiting now"
        exit 1
    fi

    if [[ -z "${SSH_PRIVATE_KEY_FILE}" ]]; then
      prompt_non_empty_str_value "Enter location of ssh private key to be used to connect to Conduit virtual VM(s): " SSH_PRIVATE_KEY_FILE
      export SSH_PRIVATE_KEY_FILE
      if [ ! -f "${SSH_PRIVATE_KEY_FILE}" ]; then
        echo "SSH_PRIVATE_KEY_FILE does not exists. Exiting now"
        exit 1
      fi
    fi
    echo "Generating PUBLIC key for Conduit VMs using selected private key ${SSH_PRIVATE_KEY_FILE}"
    ssh-keygen -y -f "${SSH_PRIVATE_KEY_FILE}" > "${SSH_PRIVATE_KEY_FILE}.pub"
    export SSH_PUBLIC_KEY_FILE="${SSH_PRIVATE_KEY_FILE}.pub"


    if [[ -z "${SESSION_AFFINITY}"  ]]; then

      PS3='Select Load balancer session affinity. Make sure your client support cookies if cookies-based is selected : '
      options=("Cookie-based" "Client IP")
      select opt in "${options[@]}"
      do
          case $opt in
              "Cookie-based")
                  export SESSION_AFFINITY="GENERATED_COOKIE"
                  break
                  ;;
              "Client IP")
                  export SESSION_AFFINITY="CLIENT_IP"
                  break
                  ;;
              *) echo "invalid option $REPLY";;
          esac
      done
    fi

    echo "Validating load balancer session affinity"
    if [[ ! $SESSION_AFFINITY =~ ^(CLIENT_IP|GENERATED_COOKIE)$ ]]; then
        echo "Invalid Chosen Session affinity: ${SESSION_AFFINITY}, expected one of [CLIENT_IP, GENERATED_COOKIE]. Exiting now"
        exit 1
    fi
}