#!/usr/bin/env bash

function install_os_requirements {
    SCRIPT_PATH="$1"
    echo "Creating custom python environment at ${SCRIPT_PATH}/env "
    python3 -m venv ${SCRIPT_PATH}/env
    source ${SCRIPT_PATH}/env/bin/activate
    pip install --upgrade pip
    pip install envtpl==0.6.0 -q --no-input
    pip install ansible==2.9.13 -q --no-input
}

function validate_os_requirements {
    if ! command -v envtpl &>/dev/null; then
      echo "Install envtpl using 'pip install envtpl' "
      exit 2
    fi

    if ! command -v ansible-playbook &>/dev/null; then
      echo "Install ansible using 'pip install ansible' "
      exit 2
    fi

    if ! command -v ssh-keygen &>/dev/null; then
      echo "Install ssh client using 'apt-get install openssh-client' "
      exit 2
    fi
}

function boolean() {
  case $1 in
  TRUE) echo true ;;
  true) echo true ;;
  FALSE) echo false ;;
  false) echo false ;;
  *)
    echo "Err: Unknown boolean value \"$1\"" 1>&2
    exit 1
    ;;
  esac
}

function yes_no_confirmation() {
  local MSG=$1
  local DEFAULT_ANSWER=$3
  if [[ -z "${NO_INPUT}" ]]; then
    while true; do
      read -p "$MSG" -r YN
      case $YN in
      [Yy]*)
        eval "$2=true"
        break
        ;;
      [Nn]*)
        eval "$2=false"
        break
        ;;
      *) echo "Please answer yes/y/Y or no/n/N." ;;
      esac
    done
  else
    if [[ -z "${DEFAULT_ANSWER}" ]]; then
      echo "ERROR: Enable to determine default value for prompt: $MSG"
      exit 6
    fi
    eval "$2=${DEFAULT_ANSWER}"
  fi
}

# Usage
#
#  prompt_non_empty_str_value "Enter deployment name: " DEPLOYMENT_NAME  "default-deployment-name"
#
function prompt_non_empty_str_value() {
  MSG=$1
  DEFAULT_ANSWER="$3"
  if [[ -z "${NO_INPUT}" ]]; then
    while true; do
      read -e -p "$MSG" -r PROMPT_VALUE
      if [[ -n "${PROMPT_VALUE}" ]]; then
        eval "$2=\"${PROMPT_VALUE}\""
        break
      else
        echo "Please enter non-empty string "
      fi
    done
  else
    if [[ -z "${DEFAULT_ANSWER}" ]]; then
      echo "ERROR: Enable to determine default value for prompt: $MSG"
      exit 6
    fi
    echo "Non-interactive mode is enabled, setting default value to ${DEFAULT_ANSWER} "
    eval "$2=\"${DEFAULT_ANSWER}\""
  fi
}

function prompt_str_value() {
  MSG=$1
  DEFAULT_ANSWER="$3"
  # if interactive mode and output variable is not defined
  if [[ -z "${NO_INPUT}" ]]; then
    read -e -p "$MSG" -r PROMPT_VALUE
    if [[ -n "${PROMPT_VALUE}" ]]; then
      eval "$2=\"${PROMPT_VALUE}\""
    else
      eval "$2=\"${DEFAULT_ANSWER}\""
    fi
  else
    if [[ -z "${DEFAULT_ANSWER}" ]]; then
      echo "ERROR: Enable to determine default value for prompt: $MSG"
      exit 6
    fi
    echo "Non-interactive mode is enabled, setting default value to ${DEFAULT_ANSWER} "
    eval "$2=\"${DEFAULT_ANSWER}\""
  fi
}

function prompt_password() {
    MSG=$1
    while true; do
        read -s -p "${MSG}: " password1
        echo
        s="${password1}"
        if [[ ${#s} -lt 12 || "$s" != *[A-Z]* || "$s" != *[a-z]* || "$s" != *[0-9]* || "$s" != *[^a-zA-Z_0-9]* ]]; then
            echo "Invalid password. The password must be at least 12 characters long and include uppercase letters, lowercase letters, digits and non-alphanumeric characters"
            continue
        fi

        read -s -p "${MSG} (again): " password2
        echo
        if [ "$password1" = "$password2" ]; then
            eval "$2=\"${password1}\""
            break
        else
            echo "Password doesn't match. Please try again"
        fi
    done
}

function parse_generic_install_parameters() {
  POSITIONAL=()
  while [[ $# -gt 0 ]]; do
    key="$1"

    case $key in
    -t | --type)
      export DEPLOYMENT_TYPE="$2"
      shift # past argument
      shift # past value
      ;;
    -n | --name)
      export DEPLOYMENT_NAME="$2"
      shift # past argument
      shift # past value
      ;;
    -r | --region | --location)
      export DEPLOYMENT_REGION="$2"
      shift # past argument
      shift # past value
      ;;
    -s | --size)
      export VM_COUNT="$2"
      shift # past argument
      shift # past value
      ;;
    *) # unknown option
      POSITIONAL+=("$1") # save it in an array for later
      shift # past argument
      ;;
    esac
  done
}

function parse_generic_conduit_parameters() {

  while [[ $# -gt 0 ]]; do
    key="$1"

    case $key in

    --domain-fqdn)
      export DEPLOYMENT_FQDN="$2"
      shift # past argument
      shift # past value
      ;;
    -t | --type)
      export DEPLOYMENT_TYPE="$2"
      shift # past argument
      shift # past value
      ;;
    -v | --version)
      export BDE_VERSION="$2"
      shift # past argument
      shift # past value
      ;;
    --file-system)
      export CONDUIT_FS_TYPE="$2"
      shift # past argument
      shift # past value
      ;;
    --docker-user)
      export DOCKER_USER="$2"
      shift # past argument
      shift # past value
      ;;
    --docker-password)
      export DOCKER_PASSWORD="$2"
      shift # past argument
      shift # past value
      ;;
    --conduit-admin-firstname)
      export CONDUIT_ADMIN_FIRSTNAME="$2"
      shift # past argument
      shift # past value
      ;;
    --conduit-admin-lastname)
      export CONDUIT_ADMIN_LASTNAME="$2"
      shift # past argument
      shift # past value
      ;;
    --conduit-admin-email)
      export CONDUIT_ADMIN_EMAIL="$2"
      shift # past argument
      shift # past value
      ;;
    --conduit-admin-password)
      export CONDUIT_ADMIN_PASSWORD="$2"
      shift # past argument
      shift # past value
      ;;
    --ssh-public-key)
      export SSH_PUBLIC_KEY_FILE="$2"
      shift # past argument
      shift # past value
      ;;
    --service-token)
      export CONDUIT_DEPLOYMENT_SERVICE_TOKEN="$2"
      shift # past argument
      shift # past value
      ;;
    *)
      shift # past
      ;;

    esac
  done

}

function prompt_generic_install_parameters() {
  if [ -z ${DEPLOYMENT_TYPE} ]; then


    echo "Available installation types are: "
    printf "\n\t1) [azure] - Creates resources and install Conduit on Azure"
    printf "\n\t2) [gcp] - Creates resources and install Conduit on Google Cloud Platform"
    printf "\n\t3) [on-premise] - Install Conduit on-premise\n\n"

    PS3='Please choose the deployment type (enter number): '
    options=("azure" "gcp" "on-premise")
    select opt in "${options[@]}"
    do
        case $opt in
            "azure")
                DEPLOYMENT_TYPE="azure"
                break
                ;;
            "gcp")
                DEPLOYMENT_TYPE="gcp"
                break
                ;;
            "on-premise")
                DEPLOYMENT_TYPE="on-premise"
                break
                ;;
            *)
                echo "invalid option $REPLY "
                ;;
        esac
    done

    export DEPLOYMENT_TYPE
  fi

}

function print_array_on_each_line() {
  local multiline=("$1")
  local arr=()
  while read -r line; do
    arr+=("$line")
  done <<<"$multiline"
  local i

  printf "\n"
  for ((i = 0; i < ${#arr[@]}; i++)); do
    printf "\t${arr[$i]}\n"
  done
  printf "\n"
}
