#!/usr/bin/env bash

SCRIPT_PATH="$( cd -- "$(dirname "$BASH_SOURCE")" >/dev/null 2>&1 ; pwd -P )"
source ${SCRIPT_PATH}/functions.sh

function set_gcloud_resources_names {
    export TAG_ALLOW_SSH="${DEPLOYMENT_NAME}-allow-ssh"
    export VNET="${DEPLOYMENT_NAME}-vnet"
    export VNET_SUBNET="${VNET}-sub"
    export VNET_CONNECTOR_NAME="${VNET}-vpc"

    export ROUTER="${DEPLOYMENT_NAME}-rt"
    export NAT="${ROUTER}-nat"
    export NAT_IP="${NAT}-ip"

    export INSTANCE_TEMPLATE="${DEPLOYMENT_NAME}-i-template"
    export INSTANCE_VM_GROUP="${DEPLOYMENT_NAME}-vm"

    export JUMPBOX_IP="${DEPLOYMENT_NAME}-jb-ip"
    export JUMPBOX_VM="${DEPLOYMENT_NAME}-jb-vm"

    export VNET_FIREWALL_ALLOW_INTERNAL="${VNET}-fw-allow-internal"
    export VNET_FIREWALL_ALLOW_EXTERNAL="${VNET}-fw-allow-external"
    export VNET_FIREWALL_ALLOW_SSH="${VNET}-fw-allow-ssh"
    export VNET_FIREWALL_HEALTH_CHECK="${VNET}-fw-health-check"

    export LB="${DEPLOYMENT_NAME}-lb"
    export LB_IP="${LB}-ip"
    export LB_HEALTH_CHECK="${LB}-hc"
    export LB_HEALTH_CHECK_SSL="${LB}-hc-ssl"
    export LB_BACKEND_SERVICE="${LB}-bs"
    export LB_BACKEND_SERVICE_SSL="${LB}-bs-ssl"
    export LB_URL_MAPS_HTTPS="${LB}-map-https"
    export LB_URL_MAPS_HTTP="${LB}-map-http"
    export LB_TARGET_PROXY_SSL="${LB}-proxy-ssl"
    export LB_TARGET_PROXY_HTTP="${LB}-proxy-http"
    export LB_TARGET_PROXY_HTTPS="${LB}-proxy-https"
    export LB_FORWARD_RULE_HTTP="${LB}-fwd-rule-http"
    export LB_FORWARD_RULE_HTTPS="${LB}-fwd-rule-https"
    export LB_FORWARD_RULE_SSL="${LB}-fwd-rule-ssl"

    export GCS_IAM_USER="${DEPLOYMENT_NAME}-iam"
}

function validate_gcloud_os_requirements {
    if ! command -v gcloud &>/dev/null; then
      echo "Install gcloud cli  https://cloud.google.com/sdk/docs/install "
      exit 2
    fi

    if ! command -v gsutil &>/dev/null; then
      echo "Install gsutil cli  https://cloud.google.com/storage/docs/gsutil_install "
      exit 2
    fi
}

function create_gcs_bucket {
    gcs_name=$1
    gsutil mb \
        -p "${GCP_PROJECT_ID}" \
        -c STANDARD \
        -l "${DEPLOYMENT_REGION}" \
        -b on \
        gs://"${gcs_name}"

    create_ret_val=$?
    return $create_ret_val
}

function create_random_gcs_name {
    bucket_variable_name=$1
    MAX_NUMBER_OF_ATTEMPTS=5
    generated_gcs_name="${DEPLOYMENT_NAME}-gcs";
    found_unique_name=false
    tries=0
    while [ $found_unique_name == "false" ] ; do
        random_string=$(openssl rand -base64 128 | tr -d -c a-z | fold -w 4 | head -n 1)
        random_name="${DEPLOYMENT_NAME}-gcs-${random_string}"

        create_gcs_bucket "${random_name}"
        calL_ret_val=$?
        if [ $calL_ret_val -ne 0 ]; then
            found_unique_name=false
        else
            generated_gcs_name="${random_name}"
            found_unique_name=true
        fi
        tries=$((tries+1))

        if [ $tries -eq $MAX_NUMBER_OF_ATTEMPTS ]; then
            echo "Maximum number of retries ({$tries}) to create Google Cloud Storage Bucket has reached. Exiting now"
            exit 1
        fi

    done
    export $bucket_variable_name="${generated_gcs_name}"
}

function validate_gcloud_parameters_are_defined {
    echo "Validating deployment name"
    if [ -z ${DEPLOYMENT_NAME} ]; then
       echo "Deployment name cannot be empty. Exiting now"
       exit 1
    fi
    if [ ${#DEPLOYMENT_NAME} -ge 20 ]; then
        echo "Deployment name is longer than 20 characters. Exiting now"
        exit 1
    fi
    NAMED_DEPLOYMENT_NAME=`gcloud compute networks list --filter="name=('${VNET}')" --format="get(NAME)"`
    if [ ! -z ${NAMED_DEPLOYMENT_NAME} ]; then
       echo "The deployment: ${DEPLOYMENT_NAME} already exists. Please Choose a different name. Exiting now"
       exit 1
    fi

    echo "Validating deployment region"
    NAMED_REGION=`gcloud compute regions list --filter="name=('${DEPLOYMENT_REGION}')" --format="get(NAME)"`
    if [ -z ${NAMED_REGION} ]; then
       echo "Invalid deployment region: ${DEPLOYMENT_REGION}. Please check available regions at https://cloud.google.com/compute/docs/regions-zones"
       exit 1
    fi

    echo "Validating google project"
    NAMED_PROJECT=`gcloud projects list --filter="PROJECT_ID=('${GCP_PROJECT_ID}')" --format="get(PROJECT_ID)"`
    if [ -z ${NAMED_PROJECT} ]; then
       echo "No such GCP project id: ${GCP_PROJECT_ID}. Exiting now"
       exit 1
    fi

    echo "Validating SSL certificate"
    if [ -z ${SSL_CERTIFICATE_NAME} ]; then
        if [ ! -f "${SSL_CERTIFICATE_PUBLIC_FILE}" ]; then
            echo "Local file ${SSL_CERTIFICATE_PUBLIC_FILE} does not exists. Exiting now"
            exit 1
        fi
        if [ ! -f "${SSL_CERTIFICATE_PRIVATE_FILE}" ]; then
            echo "Local file ${SSL_CERTIFICATE_PRIVATE_FILE} does not exists. Exiting now"
            exit 1
        fi
    else
        NAMED_CERTIFICATE=`gcloud compute ssl-certificates list --filter="name=('${SSL_CERTIFICATE_NAME}')" --format="get(NAME)"`
        if [ -z ${NAMED_CERTIFICATE} ]; then
           echo "No such SSL certificate registered on GCP: ${SSL_CERTIFICATE_NAME}. Exiting now"
           exit 1
        fi
    fi

    echo "Validating number of virtual machines"
    if [[ $VM_COUNT =~ '!^[0-9]+$' ]] ; then
       echo "Invalid input for the number of virtual machines: ${VM_COUNT}. Exiting now"
       exit 1
    fi

    echo "Validating conduit file system type"
    if [[ ! $CONDUIT_FS_TYPE =~ ^(file|hdfs|abfs|s3|gcs)$ ]]; then
        echo "Invalid Chosen File System type: ${CONDUIT_FS_TYPE}, expected one of [file|hdfs|abfs|s3|gcs]. Exiting now"
        exit 1
    fi

    echo "Validating RSA access keys"
    if [ ! -f "${SSH_PRIVATE_KEY_FILE}" ]; then
        echo "SSH private key does not exists. Exiting now"
        exit 1
    fi

    echo "Validating load balancer session affinity"
    if [[ ! $SESSION_AFFINITY =~ ^(NONE|CLIENT_IP|GENERATED_COOKIE)$ ]]; then
        echo "Invalid Chosen Session affinity: ${SESSION_AFFINITY}, expected one of [NONE, CLIENT_IP, GENERATED_COOKIE]. Exiting now"
        exit 1
    fi

    echo "Validating conduit deployment service authentication token"
    if [ -z "${CONDUIT_DEPLOYMENT_SERVICE_TOKEN}" ]; then
        echo "Conduit deployment service authentication token cannot be empty. Exiting now"
        exit 1
    fi
}

function prompt_deployment_name_value {
    while true; do
        read -e -p "Enter the deployment name: " -r PV
        if [[ ${#PV} -le 32 ]]; then
            if [[ -n "${PV}" ]]; then
                if [[ "$PV" == *[a-z]* || "$PV" == *[0-9]* || "$PV" == *-* ]]; then
                    if [[ "$PV" == [a-z]* ]]; then
                        if [[ "$PV" == *[a-z] || "$PV" == *[0-9] ]]; then
                            eval "$1=\"${PV}\""
                            break
                        else
                            echo "Deployment name must end with a lowercase letter or a number"
                        fi
                    else
                        echo "Deployment name must start with a lowercase letter"
                    fi
                else
                    echo "Deployment name must consist of lowercase letters (a-z), numbers and hyphens"
                fi
            else
                echo "Please enter non-empty string "
            fi
        else
            echo "Deployment name may be at most 32 characters"
        fi
    done
}


function load_gcloud_parameters_input {
    if [ -z ${VM_COUNT} ]; then
        echo "INFO: Currently we support only 3 instances via automated installation script"
        prompt_str_value "Enter the number of instances to deploy [minimum 3]: " VM_COUNT 3
        export VM_COUNT
    fi

    if [ -z ${GCP_PROJECT_ID} ]; then
        AVAILABLE_PROJECTS=`gcloud projects list --format="get(PROJECT_ID)"`
        echo "Choose one of the following available projects:"
        print_array_on_each_line "${AVAILABLE_PROJECTS}"
        DEFAULT_PROJECT=`gcloud config get-value project`
        prompt_str_value "Enter the Google project id. Default [${DEFAULT_PROJECT}]: " GCP_PROJECT_ID "${DEFAULT_PROJECT}"
        export GCP_PROJECT_ID
    fi

    if [ -z ${DEPLOYMENT_NAME} ]; then
        echo "INFO. The deployment name needs to be unique across entire selected (${GCP_PROJECT_ID}) Google Cloud Project"
        echo "INFO. The deployment name must contain only letters, numbers or the hyphen characters"
        prompt_deployment_name_value DEPLOYMENT_NAME
        export DEPLOYMENT_NAME
    fi

    if [ -z ${DEPLOYMENT_REGION} ]; then
        echo "Loading Google Cloud regions..."
        AVAILABLE_REGIONS=`gcloud compute regions list --format="get(NAME)" --filter="name:us-* OR name:northamerica-*"`
        echo "For other available regions please see Google documentation at https://cloud.google.com/compute/docs/regions-zones"
        echo "Ensure if you have at least the following available resources for the region you want to use: 16 CPUs, 250GB SSD disk space, 3 public IPs, 1 URL map, 1 Router, 1 SSL certificate, 1 network."
        echo "Choose one of the following available regions:"
        print_array_on_each_line "${AVAILABLE_REGIONS}"
        prompt_non_empty_str_value "Enter the deployment region: " DEPLOYMENT_REGION
        export DEPLOYMENT_REGION
    fi

    if [ -z ${SSL_CERTIFICATE_NAME} ]; then
        while true; do
        read -p "Do you have Google cloud registered SSL certificate? [Yes|No]:" yn
        case $yn in
            [Yy]* )
                prompt_non_empty_str_value "Enter the SSL certificate name: " SSL_CERTIFICATE_NAME;
                export SSL_CERTIFICATE_NAME
            break;;
            [Nn]* )
                prompt_non_empty_str_value "Enter the public SSL certificate file path: " SSL_CERTIFICATE_PUBLIC_FILE;
                prompt_non_empty_str_value "Enter the private SSL certificate file path: " SSL_CERTIFICATE_PRIVATE_FILE;
                export SSL_CERTIFICATE_PUBLIC_FILE
                export SSL_CERTIFICATE_PRIVATE_FILE
            break;;
            * ) echo "Please answer yes or no.";;
        esac
    done
    fi


    if [ -z ${DEPLOYMENT_FQDN} ]; then
        prompt_non_empty_str_value "Enter the Fully Qualified Domain Name (FQDN) of Conduit deployment: " DEPLOYMENT_FQDN
        export DEPLOYMENT_FQDN
    fi

    if [ -z ${CONDUIT_FS_TYPE} ]; then
        echo "Conduit uses  filesystem cache to improve performance of certain types of queries. It's possible to use cloud-based storage for this purpose"
        echo "Deployment script will create a dedicated cloud storage to configure Conduit filesystem cache."
        printf "\n\thdfs - Hadoop Distributed File System"
        printf "\n\tgcs - Google Cloud Storage"
        printf "\n\ts3 - Amazon Simple Storage Service"
        printf "\n\tabfs - Azure Blob File System\n\n"
        PS3='Choose filesystem type for Conduit Parquet Store [Recommended gcs] (enter number): '
        options=("hdfs" "gcs" "s3" "abfs")
        select opt in "${options[@]}"
        do
            case $opt in
                "hdfs")
                    CONDUIT_FS_TYPE="hdfs"
                    break
                    ;;
                "gcs")
                    CONDUIT_FS_TYPE="gcs"
                    break
                    ;;
                "s3")
                    CONDUIT_FS_TYPE="s3"
                    break
                    ;;
                "abfs")
                    CONDUIT_FS_TYPE="abfs"
                    break
                    ;;
                *)
                    echo "Invalid option $REPLY, using default 'gcs'  "
                    CONDUIT_FS_TYPE="gcs"
                    ;;
            esac
        done

        export CONDUIT_FS_TYPE
    fi


    if [ 's3' == ${CONDUIT_FS_TYPE} ]; then
        echo "Using File System on Amazon Simple Storage Service"
        prompt_non_empty_str_value "Enter the AWS S3 Bucket Name: " AWS_S3_BUCKET_NAME
        export AWS_S3_BUCKET_NAME
        prompt_non_empty_str_value "Enter the AWS S3 Bucket Access Key: " AWS_S3_BUCKET_ACCESS_KEY
        export AWS_S3_BUCKET_ACCESS_KEY
        prompt_non_empty_str_value "Enter the AWS S3 Bucket Secret Key: " AWS_S3_BUCKET_SECRET_KEY
        export AWS_S3_BUCKET_SECRET_KEY
    elif [ 'abfs' == ${CONDUIT_FS_TYPE} ]; then
        echo "Using File System on Azure Blob File System"
        prompt_non_empty_str_value "Enter the Azure Blob Storage Account Name: " AZURE_STORAGE_NAME
        export AZURE_STORAGE_NAME
        prompt_non_empty_str_value "Enter the Azure Blob Container Name: " AZURE_CONTAINER_NAME
        export AZURE_CONTAINER_NAME
        prompt_non_empty_str_value "Enter the Azure Blob Access Key: " AZURE_CONTAINER_ACCESS_KEY
        export AZURE_CONTAINER_ACCESS_KEY
    fi


    if [ -z ${CONDUIT_ADMIN_FIRSTNAME} ]; then
        prompt_non_empty_str_value "Enter the Conduit admin first name: " CONDUIT_ADMIN_FIRSTNAME
        export CONDUIT_ADMIN_FIRSTNAME
    fi

    if [ -z ${CONDUIT_ADMIN_LASTNAME} ]; then
        prompt_non_empty_str_value "Enter the Conduit admin last name: " CONDUIT_ADMIN_LASTNAME
        export CONDUIT_ADMIN_LASTNAME
    fi

    if [ -z ${CONDUIT_ADMIN_EMAIL} ]; then
        prompt_non_empty_str_value "Enter the Conduit admin email address: " CONDUIT_ADMIN_EMAIL
        export CONDUIT_ADMIN_EMAIL
    fi

    if [ -z ${CONDUIT_ADMIN_PASSWORD} ]; then
        prompt_password "Enter the Conduit admin email password: " CONDUIT_ADMIN_PASSWORD
        export CONDUIT_ADMIN_PASSWORD
        echo
    fi

    if [ -z ${SSH_PRIVATE_KEY_FILE} ]; then
        prompt_non_empty_str_value "Enter the ssh private key file path: " SSH_PRIVATE_KEY_FILE
        export SSH_PRIVATE_KEY_FILE
    fi

    if [ -z ${SESSION_AFFINITY} ]; then
        echo "Load Balancer session affinity is responsible for traffic balancing between Conduit machines"
        echo "[Info] The Session Affinity 'GENERATED_COOKIE' is supported only for the Console (https). The thrift port (5222) will be defaulted to 'CLIENT_IP' session affinity."
        echo "More about Session Affinity: https://cloud.google.com/load-balancing/docs/backend-service#session_affinity"
        printf "\n\t 1) CLIENT_IP - 2-tuple hash of Source IP address and Destination IP address."
        printf "\n\t 2) GENERATED_COOKIE - The load balancer issues a cookie on the first request. "
        printf "\n\t 3) NONE - Session affinity is disabled\n\n"
        PS3='Choose Session Affinity for Conduit Load Balancer [Recommended CLIENT_IP] (enter number): '
        options=("CLIENT_IP" "GENERATED_COOKIE" "NONE")
        select opt in "${options[@]}"
        do
            case $opt in
                "CLIENT_IP")
                    SESSION_AFFINITY="CLIENT_IP"
                    break
                    ;;
                "GENERATED_COOKIE")
                    SESSION_AFFINITY="GENERATED_COOKIE"
                    break
                    ;;
                "NONE")
                    SESSION_AFFINITY="NONE"
                    break
                    ;;
                *)
                    echo "invalid option $REPLY, using default CLIENT_IP  "
                    SESSION_AFFINITY="CLIENT_IP"
                    break
                    ;;
            esac
        done

        export SESSION_AFFINITY
    fi
}

function validate_gcloud_uninstall_parameters_are_defined {
    if [ -z ${DEPLOYMENT_NAME} ]; then
       echo "Deployment name cannot be empty. Exiting now"
       exit 1
    fi
    if [ ${#DEPLOYMENT_NAME} -ge 24 ]; then
        echo "Deployment name is longer than 24 characters. Exiting now"
        exit 1
    fi

    NAMED_REGION=`gcloud compute regions list --filter="name=('${DEPLOYMENT_REGION}')" --format="get(NAME)"`
    if [ -z ${NAMED_REGION} ]; then
       echo "Invalid deployment region: ${DEPLOYMENT_REGION}. Please check available regions at https://cloud.google.com/compute/docs/regions-zones"
       exit 1
    fi

    NAMED_PROJECT=`gcloud projects list --filter="PROJECT_ID=('${GCP_PROJECT_ID}')" --format="get(PROJECT_ID)"`
    if [ -z ${NAMED_PROJECT} ]; then
       echo "No such GCP project id: ${GCP_PROJECT_ID}. Exiting now"
       exit 1
    fi
}


function load_gcloud_uninstall_parameters_input() {
  if [ -z ${GCP_PROJECT_ID} ]; then
    AVAILABLE_PROJECTS=$(gcloud projects list --format="get(PROJECT_ID)")
    echo "Choose one of the following available projects:"
    print_array_on_each_line "${AVAILABLE_PROJECTS}"
    DEFAULT_PROJECT=$(gcloud config get-value project)
    prompt_str_value "Enter the Google project id. Default [${DEFAULT_PROJECT}]: " GCP_PROJECT_ID $DEFAULT_PROJECT
    export GCP_PROJECT_ID
  fi

  if [ -z ${DEPLOYMENT_NAME} ]; then
    prompt_non_empty_str_value "Enter the deployment name: " DEPLOYMENT_NAME
    export DEPLOYMENT_NAME
  fi

  if [ -z ${DEPLOYMENT_REGION} ]; then
    echo "Loading Google Cloud regions..."
    AVAILABLE_REGIONS=$(gcloud compute regions list --format="get(NAME)" --filter="name:us-* OR name:northamerica-*")
    echo "For other available regions please see Google documentation at https://cloud.google.com/compute/docs/regions-zones"
    echo ""
    echo "Choose one of the following available regions:"
    print_array_on_each_line "${AVAILABLE_REGIONS}"
    prompt_non_empty_str_value "Enter the deployment region: " DEPLOYMENT_REGION
    export DEPLOYMENT_REGION
  fi
}

function load_gcloud_parameters() {

    POSITIONAL=()
    while [[ $# -gt 0 ]]; do
        key="$1"

        case $key in
         -p | --project)
          export GCP_PROJECT_ID="$2"
          shift # past argument
          shift # past value
          ;;
         -c|--certificate)
          export SSL_CERTIFICATE_NAME="$2"
          shift # past argument
          shift # past value
          ;;
          --certificate-public)
          export SSL_CERTIFICATE_PUBLIC_FILE="$2"
          shift # past argument
          shift # past value
          ;;
          --certificate-private)
          export SSL_CERTIFICATE_PRIVATE_FILE="$2"
          shift # past argument
          shift # past value
          ;;
          --ssh-private-key)
          export SSH_PRIVATE_KEY_FILE="$2"
          shift # past argument
          shift # past value
          ;;
          --session-affinity)
          export SESSION_AFFINITY="$2"
          shift # past argument
          shift # past value
          ;;
          --service-token)
          export CONDUIT_DEPLOYMENT_SERVICE_TOKEN="$2"
          shift # past argument
          shift # past value
          ;;
        *) # unknown option
          POSITIONAL+=("$1") # save it in an array for later
          shift # past argument
          ;;
      esac
    done
}


function load_gcloud_uninstall_parameters() {
  while [[ $# -gt 0 ]]; do
    key="$1"

    case $key in
    -p | --project)
      export GCP_PROJECT_ID="$2"
      shift # past argument
      shift # past value
      ;;
    *) # unknown option
      shift # past argument
      ;;
    esac
  done
}
